from __future__ import unicode_literals

import datetime

from django.core.validators import RegexValidator
from django.db import transaction
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers, exceptions
from waldur_ansible.common import serializers as common_serializers

from waldur_core.core import models as core_models, serializers as core_serializers
from waldur_core.structure import permissions as structure_permissions, serializers as structure_serializers, models as structure_models
from . import models, utils

REQUEST_TYPES_PLAIN_NAMES = {
    models.PythonManagement: 'overall',
    models.PythonManagementInitializeRequest: 'initialization',
    models.PythonManagementSynchronizeRequest: 'synchronization',
    models.PythonManagementFindVirtualEnvsRequest: 'virtual_envs_search',
    models.PythonManagementFindInstalledLibrariesRequest: 'installed_libraries_search',
    models.PythonManagementDeleteRequest: 'python_management_deletion',
    models.PythonManagementDeleteVirtualEnvRequest: 'virtual_environment_deletion',
}

directory_and_library_allowed_pattern = '^[a-zA-Z0-9\-_]+$'


class InstalledPackageSerializer(core_serializers.AugmentedSerializerMixin, serializers.HyperlinkedModelSerializer):
    name = serializers.RegexField(directory_and_library_allowed_pattern)

    class Meta(object):
        model = models.InstalledLibrary
        fields = ('name', 'version', 'uuid',)
        read_only_fields = ('uuid',)
        extra_kwargs = {
            'url': {'lookup_field': 'uuid'},
        }


class VirtualEnvironmentSerializer(core_serializers.AugmentedSerializerMixin, serializers.HyperlinkedModelSerializer):
    name = serializers.RegexField(directory_and_library_allowed_pattern)
    installed_libraries = InstalledPackageSerializer(many=True)

    class Meta(object):
        model = models.VirtualEnvironment
        fields = ('name', 'uuid', 'installed_libraries', 'jupyter_hub_global',)
        read_only_fields = ('uuid',)
        extra_kwargs = {
            'url': {'lookup_field': 'uuid'},
        }


class PythonManagementRequestMixin(common_serializers.BaseApplicationSerializer):
    request_type = serializers.SerializerMethodField()
    state = serializers.SerializerMethodField()
    output = serializers.SerializerMethodField()

    class Meta(object):
        model = NotImplemented
        fields = ('uuid', 'output', 'state', 'created', 'modified', 'request_type',)
        read_only_fields = ('uuid', 'output', 'state', 'created', 'modified', 'request_type',)
        extra_kwargs = {
            'url': {'lookup_field': 'uuid'},
        }

    def get_output(self, obj):
        if self.context.get('select_output'):
            return obj.output
        else:
            return None

    def get_request_type(self, obj):
        return REQUEST_TYPES_PLAIN_NAMES.get(type(obj))

    def get_state(self, obj):
        return obj.human_readable_state


class PythonManagementInitializeRequestSerializer(PythonManagementRequestMixin):
    class Meta(PythonManagementRequestMixin.Meta):
        model = models.PythonManagementInitializeRequest


class PythonManagementFindVirtualEnvsRequestSerializer(PythonManagementRequestMixin):
    class Meta(PythonManagementRequestMixin.Meta):
        model = models.PythonManagementFindVirtualEnvsRequest


class PythonManagementFindInstalledLibrariesRequestSerializer(PythonManagementRequestMixin):
    class Meta(PythonManagementRequestMixin.Meta):
        model = models.PythonManagementFindInstalledLibrariesRequest
        fields = PythonManagementRequestMixin.Meta.fields + ('virtual_env_name',)


class PythonManagementDeleteRequestSerializer(PythonManagementRequestMixin):
    class Meta(PythonManagementRequestMixin.Meta):
        model = models.PythonManagementDeleteRequest


class PythonManagementDeleteVirtualEnvRequestSerializer(PythonManagementRequestMixin):
    class Meta(PythonManagementRequestMixin.Meta):
        model = models.PythonManagementDeleteVirtualEnvRequest
        fields = PythonManagementRequestMixin.Meta.fields + ('virtual_env_name',)


class PythonManagementSynchronizeRequestSerializer(PythonManagementRequestMixin):
    libraries_to_install = serializers.JSONField(default=dict)
    libraries_to_remove = serializers.JSONField(default=dict)

    class Meta(PythonManagementRequestMixin.Meta):
        model = models.PythonManagementSynchronizeRequest
        fields = PythonManagementRequestMixin.Meta.fields \
            + ('libraries_to_install', 'libraries_to_remove', 'virtual_env_name')


class PythonManagementSerializer(
        common_serializers.BaseApplicationSerializer,
        structure_serializers.PermissionFieldFilteringMixin):
    REQUEST_IN_PROGRESS_STATES = (core_models.StateMixin.States.CREATION_SCHEDULED, core_models.StateMixin.States.CREATING)

    state = serializers.SerializerMethodField()
    virtual_environments = VirtualEnvironmentSerializer(many=True)
    virtual_envs_dir_path = serializers.CharField(max_length=255, validators=[
        RegexValidator(
            regex=directory_and_library_allowed_pattern,
            message=_('Virtual environments root directory has invalid format!'),
        ),
    ])
    name = serializers.SerializerMethodField()
    type = serializers.SerializerMethodField()

    instance = core_serializers.GenericRelatedField(
        related_models=structure_models.ResourceMixin.get_all_models(), required=False)
    instance_name = serializers.ReadOnlyField(source='instance.name')

    class Meta(object):
        model = models.PythonManagement
        fields = ('url', 'uuid', 'virtual_envs_dir_path', 'system_user',
                  'state', 'created', 'modified', 'virtual_environments', 'python_version', 'name', 'type', 'instance', 'instance_name',)
        read_only_fields = ('request_states', 'created', 'modified', 'python_version', 'type', 'name', 'url',)
        extra_kwargs = {
            'url': {'lookup_field': 'uuid'},
        }

    def get_name(self, python_management):
        instance_name = python_management.instance.name if python_management.instance else 'removed instance'
        return 'Python Management - %s - %s' % (instance_name, python_management.virtual_envs_dir_path)

    def get_type(self, python_management):
        return 'python_management'

    def get_filtered_field_names(self):
        return 'project'

    def get_state(self, python_management):
        states = []
        initialize_request = utils.execute_safely(
            lambda: models.PythonManagementInitializeRequest.objects.filter(python_management=python_management).latest('id'))
        if initialize_request and self.is_in_progress_or_errored(initialize_request):
            return [self.build_state(initialize_request)]

        states.extend(self.get_request_state(
            utils.execute_safely(lambda: models.PythonManagementDeleteRequest.objects.filter(python_management=python_management).latest('id'))))
        states.extend(
            self.get_request_state(
                utils.execute_safely(
                    lambda: models.PythonManagementDeleteVirtualEnvRequest.objects.filter(python_management=python_management).latest('id'))))
        states.extend(self.build_search_requests_states(python_management))
        states.extend(self.build_states_from_last_group_of_the_request(python_management, models.PythonManagementSynchronizeRequest))

        if not states:
            return core_models.StateMixin(state=core_models.StateMixin.States.OK).human_readable_state
        else:
            creation_scheduled_state = core_models.StateMixin(state=core_models.StateMixin.States.CREATION_SCHEDULED).human_readable_state
            creating_state = core_models.StateMixin(state=core_models.StateMixin.States.CREATING).human_readable_state
            erred_state = core_models.StateMixin(state=core_models.StateMixin.States.ERRED).human_readable_state
            if creating_state in states:
                return creating_state
            elif creation_scheduled_state in states:
                return creation_scheduled_state
            elif erred_state in states:
                return erred_state

    def build_search_requests_states(self, python_management):
        states = []
        states.extend(
            self.get_request_state(
                utils.execute_safely(
                    lambda: models.PythonManagementFindVirtualEnvsRequest.objects
                    .filter(python_management=python_management).latest('id'))))
        states.extend(self.build_states_from_last_group_of_the_request(python_management, models.PythonManagementFindInstalledLibrariesRequest))
        return states

    def get_request_state(self, request):
        if request and self.is_in_progress_or_errored(request):
            return [self.build_state(request)]
        else:
            return []

    def build_states_from_last_group_of_the_request(self, python_management, request_class):
        states = []
        requests = request_class.objects.filter(python_management=python_management).order_by('-id')
        last_request_group = self.get_last_requests_group(requests)
        for request in last_request_group:
            if self.is_in_progress_or_errored(request):
                states.append(self.build_state(request))
        return states

    def get_last_requests_group(self, requests):
        last_request_group = []

        last_request_time = None
        for request in requests:
            if not last_request_time:
                last_request_time = request.created - datetime.timedelta(minutes=1)
            if request.created < last_request_time:
                break
            last_request_group.append(request)

        return last_request_group

    def is_in_progress_or_errored(self, request):
        return request.state in PythonManagementSerializer.REQUEST_IN_PROGRESS_STATES \
            or request.state == core_models.StateMixin.States.ERRED

    def build_state(self, request, state=None):
        request_state = state if state else request
        return request_state.human_readable_state

    @transaction.atomic
    def create(self, validated_data):
        python_management = models.PythonManagement(
            user=validated_data.get('user'),
            virtual_envs_dir_path=validated_data.get('virtual_envs_dir_path'),
            instance=validated_data.get('instance'),
            project=validated_data.get('instance').service_project_link.project,
            python_version='3',
            system_user=validated_data.get('system_user'))
        python_management.save()
        return python_management

    def validate(self, attrs):
        super(PythonManagementSerializer, self).validate(attrs)
        if not self.instance:
            attrs['user'] = self.context['request'].user
            self.check_resource_type(attrs)

        self.check_project_permissions(attrs)
        return attrs

    def check_resource_type(self, attrs):
        if not issubclass(type(attrs['instance']), structure_models.VirtualMachine):
            raise exceptions.ValidationError(_('Please specify a virtual machine, not just any resource.'))

    def check_project_permissions(self, attrs):
        if self.instance:
            project = self.instance.project
        else:
            project = attrs['instance'].service_project_link.project

        if not structure_permissions._has_admin_access(self.context['request'].user, project):
            raise exceptions.PermissionDenied()


class CachedRepositoryPythonLibrarySerializer(
        core_serializers.AugmentedSerializerMixin,
        serializers.HyperlinkedModelSerializer):
    class Meta(object):
        model = models.CachedRepositoryPythonLibrary
        fields = ('name', 'uuid')
        extra_kwargs = {
            'url': {'lookup_field': 'uuid'},
        }



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('invoices', '0030_json_details_on_genericinvoiceitem'),
    ]

    operations = [
        migrations.RenameModel('GenericInvoiceItem', 'InvoiceItem')
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-07-14 00:19
from __future__ import unicode_literals

from decimal import Decimal
import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0009_offering_article_code'),
    ]

    operations = [
        migrations.AddField(
            model_name='offering',
            name='unit',
            field=models.CharField(
                choices=[(b'month', 'Per month'),
                         (b'half_month', 'Per half month'),
                         (b'day', 'Per day'),
                         (b'hour', 'Per hour'),
                         (b'quantity', 'Quantity')], default=b'day', max_length=30),
        ),
        migrations.AddField(
            model_name='offering',
            name='unit_price',
            field=models.DecimalField(decimal_places=7, default=0, max_digits=22,
                                      validators=[django.core.validators.MinValueValidator(Decimal('0'))]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def import_quotas(apps, schema_editor):
    from waldur_mastermind.analytics.utils import import_daily_usage

    import_daily_usage()


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('analytics', '0001_initial'),
        ('quotas', '0001_squashed_0004'),
        ('reversion', '0001_squashed_0004_auto_20160611_1202'),
    ]

    operations = [
        migrations.RunPython(import_quotas),
    ]



from django.conf.urls import url

from . import views


def register_in(router):
    router.register(r'openstacktenant', views.OpenStackServiceViewSet, base_name='openstacktenant')
    router.register(r'openstacktenant-service-project-link', views.OpenStackServiceProjectLinkViewSet,
                    base_name='openstacktenant-spl')
    router.register(r'openstacktenant-images', views.ImageViewSet, base_name='openstacktenant-image')
    router.register(r'openstacktenant-flavors', views.FlavorViewSet, base_name='openstacktenant-flavor')
    router.register(r'openstacktenant-floating-ips', views.FloatingIPViewSet, base_name='openstacktenant-fip')
    router.register(r'openstacktenant-security-groups', views.SecurityGroupViewSet, base_name='openstacktenant-sgp')
    router.register(r'openstacktenant-volumes', views.VolumeViewSet, base_name='openstacktenant-volume')
    router.register(r'openstacktenant-snapshots', views.SnapshotViewSet, base_name='openstacktenant-snapshot')
    router.register(r'openstacktenant-instance-availability-zones', views.InstanceAvailabilityZoneViewSet,
                    base_name='openstacktenant-instance-availability-zone')
    router.register(r'openstacktenant-instances', views.InstanceViewSet, base_name='openstacktenant-instance')
    router.register(r'openstacktenant-backups', views.BackupViewSet, base_name='openstacktenant-backup')
    router.register(r'openstacktenant-backup-schedules', views.BackupScheduleViewSet,
                    base_name='openstacktenant-backup-schedule')
    router.register(r'openstacktenant-snapshot-schedules', views.SnapshotScheduleViewSet,
                    base_name='openstacktenant-snapshot-schedule')
    router.register(r'openstacktenant-subnets', views.SubNetViewSet, base_name='openstacktenant-subnet')
    router.register(r'openstacktenant-networks', views.NetworkViewSet, base_name='openstacktenant-network')
    router.register(r'openstacktenant-volume-types', views.VolumeTypeViewSet, base_name='openstacktenant-volume-type')
    router.register(r'openstacktenant-volume-availability-zones', views.VolumeAvailabilityZoneViewSet,
                    base_name='openstacktenant-volume-availability-zone')


urlpatterns = [
    url(r'^api/openstack-shared-settings-instances/$', views.SharedSettingsInstances.as_view()),
    url(r'^api/openstack-shared-settings-customers/$', views.SharedSettingsCustomers.as_view()),
]



from __future__ import unicode_literals

from collections import OrderedDict

from rest_framework import pagination
from rest_framework.response import Response
from rest_framework.utils.urls import remove_query_param, replace_query_param


class LinkHeaderPagination(pagination.PageNumberPagination):
    page_size_query_param = 'page_size'
    max_page_size = 300

    def get_paginated_response(self, data):
        link_candidates = OrderedDict((
            ('first', self.get_first_link),
            ('prev', self.get_previous_link),
            ('next', self.get_next_link),
            ('last', self.get_last_link),
        ))

        link = ', '.join(
            '<%s>; rel="%s"' % (get_link(), rel)
            for rel, get_link in link_candidates.items()
            if get_link()
        )

        headers = {
            'X-Result-Count': self.page.paginator.count,
            'Link': link,
        }

        return Response(data, headers=headers)

    def get_first_link(self):
        url = self.request.build_absolute_uri()
        return remove_query_param(url, self.page_query_param)

    def get_last_link(self):
        url = self.request.build_absolute_uri()
        page_number = self.page.paginator.page_range[-1]
        if page_number == 1:
            return remove_query_param(url, self.page_query_param)
        return replace_query_param(url, self.page_query_param, page_number)


class UnlimitedLinkHeaderPagination(LinkHeaderPagination):
    """
    A hackish paginator for cases when calculating a queryset to display is an expensive query and if
    once the result set is known, it's cheaper to serialize larger output.

    Should be used only as a temporary workaround!
    """
    page_size = None



from waldur_core.core import executors as core_executors

from . import tasks


class InvoicePDFCreateExecutor(core_executors.BaseExecutor):

    @classmethod
    def get_task_signature(cls, invoice, serialized_invoice, **kwargs):
        return tasks.create_invoice_pdf.si(serialized_invoice)



import logging
# patched with xmlrpc.monkey_patch() below
import xmlrpclib  # nosec

from celery import shared_task
from defusedxml import xmlrpc
from django.conf import settings
from django.core.validators import MaxValueValidator

from . import models

xmlrpc.monkey_patch()
logger = logging.getLogger(__name__)


@shared_task(name='waldur_ansible.sync_pip_libraries')
def sync_pip_libraries():
    """
    This task is used by Celery beat in order to periodically
    schedule available PIP libraries synchronization.
    """
    schedule_sync()


def schedule_sync():
    """
    This function calls task only if it is enabled. It is optionally enabled due to its heavy processing nature.
    """
    if not settings.WALDUR_PYTHON_MANAGEMENT.get('SYNC_PIP_PACKAGES_TASK_ENABLED'):
        return

    _sync_pip_libraries.apply_async(countdown=10)


@shared_task()
def _sync_pip_libraries():
    """
    This task is called asynchronously by Celery beat schedule.
    """
    logger.info('Started synching PIP packages.')
    client = xmlrpclib.ServerProxy('https://pypi.python.org/pypi')
    actual_repository_packages = client.list_packages()
    previously_cached_packages = models.CachedRepositoryPythonLibrary.objects.values_list('name', flat=True)

    delete_removed_libraries(actual_repository_packages, previously_cached_packages)

    persist_new_libraries(actual_repository_packages, previously_cached_packages)


def persist_new_libraries(actual_repository_packages, previously_cached_packages):
    current_batch = []
    for library_name in actual_repository_packages:
        if library_name not in previously_cached_packages:
            if len(current_batch) == settings.WALDUR_PYTHON_MANAGEMENT.get('SYNC_PIP_PACKAGES_BATCH_SIZE'):
                try:
                    models.CachedRepositoryPythonLibrary.objects.bulk_create(current_batch)
                except MaxValueValidator:
                    logger.exception('Pip backend could not save "%s" python library: name is too long.', library_name)
                current_batch = []

            current_batch.append(models.CachedRepositoryPythonLibrary(name=library_name))

    if current_batch:
        models.CachedRepositoryPythonLibrary.objects.bulk_create(current_batch)


def delete_removed_libraries(actual_repository_packages, previously_cached_packages):
    library_names_to_delete = []
    for previously_cached_package in previously_cached_packages:
        if previously_cached_package not in actual_repository_packages:
            library_names_to_delete.append(previously_cached_package)
    if library_names_to_delete:
        models.CachedRepositoryPythonLibrary.objects.filter(name__in=library_names_to_delete).delete()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-07-09 07:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openstack', '0041_tenant_default_volume_type_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='subnet',
            name='disable_gateway',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from decimal import Decimal
import django.core.validators


def migrate_package_component_price(apps, schema_editor):
    PackageComponent = apps.get_model('packages', 'PackageComponent')

    for package_component in PackageComponent.objects.all():
        package_component.price *= 24
        package_component.save()


class Migration(migrations.Migration):

    dependencies = [
        ('packages', '0006_trial_packagetemplate'),
    ]

    operations = [
        migrations.AlterField(
            model_name='packagecomponent',
            name='price',
            field=models.DecimalField(default=0, verbose_name='Price per unit per day', max_digits=14, decimal_places=10, validators=[django.core.validators.MinValueValidator(Decimal('0'))]),
        ),
        migrations.RunPython(migrate_package_component_price, elidable=True),
    ]



from django.apps import AppConfig


class RancherConfig(AppConfig):
    name = 'waldur_rancher'
    verbose_name = 'Rancher'
    service_name = 'Rancher'

    def ready(self):
        from waldur_core.structure import SupportedServices

        from .backend import RancherBackend

        SupportedServices.register_backend(RancherBackend)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-12-21 13:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0048_add_request_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='offering',
            name='billable',
            field=models.BooleanField(default=True, help_text='Purchase and usage is invoiced.'),
        ),
    ]



from waldur_ansible.python_management import executors, models, utils

from . import locking_service


class InstalledLibrariesExtractedInformationHandler(object):
    def handle_extracted_information(self, request, lines_post_processor):
        self.persist_installed_libraries_in_db(
            request.python_management,
            request.virtual_env_name,
            lines_post_processor.installed_libraries_after_modifications)

    def persist_installed_libraries_in_db(self, python_management, virtual_environment_name, existing_packages):
        virtual_environment = utils.execute_safely(
            lambda: python_management.virtual_environments.get(name=virtual_environment_name))
        if virtual_environment and not existing_packages:
            virtual_environment.delete()
            return

        if not virtual_environment:
            virtual_environment = models.VirtualEnvironment(name=virtual_environment_name, python_management=python_management)
            virtual_environment.save()

        persisted_packages = virtual_environment.installed_libraries.all()

        self.save_newly_installed_libraries(
            existing_packages, persisted_packages, virtual_environment)

        self.delete_removed_libs(existing_packages, persisted_packages)

    def delete_removed_libs(self, existing_packages, persisted_packages):
        for persisted_package in persisted_packages:
            if not self.is_package_present(persisted_package, existing_packages):
                persisted_package.delete()

    def save_newly_installed_libraries(self, existing_packages, persisted_installed_libraries, virtual_environment):
        for installed_package in existing_packages:
            if not self.is_package_present(installed_package, persisted_installed_libraries):
                models.InstalledLibrary.objects.create(
                    name=installed_package.name, version=installed_package.version,
                    virtual_environment=virtual_environment)

    def is_package_present(self, package, packages_list):
        for p in packages_list:
            if package.name == p.name and package.version == p.version:
                return True
        return False


class PythonManagementDeletionRequestExtractedInformationHandler(object):
    def handle_extracted_information(self, request, lines_post_processor):
        request.python_management.delete()


class PythonManagementDeleteVirtualEnvExtractedInformationHandler(object):
    def handle_extracted_information(self, request, lines_post_processor):
        request.python_management.virtual_environments.get(name=request.virtual_env_name).delete()


class PythonManagementFindVirtualEnvsRequestExtractedInformationHandler(object):
    def handle_extracted_information(self, request, lines_post_processor):
        for persisted_virtual_env in request.python_management.virtual_environments.all():
            if persisted_virtual_env not in lines_post_processor.installed_virtual_environments:
                persisted_virtual_env.delete()

        locking_service.PythonManagementBackendLockingService.handle_on_processing_finished(request)

        for virtual_env_name in lines_post_processor.installed_virtual_environments:
            find_libs_request = models.PythonManagementFindInstalledLibrariesRequest.objects.create(
                python_management=request.python_management, virtual_env_name=virtual_env_name)
            executors.PythonManagementRequestExecutor.execute(find_libs_request, async=True)


class InitializationRequestExtractedInformationHandler(object):
    def handle_extracted_information(self, request, lines_post_processor):
        python_management = request.python_management
        python_management.python_version = lines_post_processor.python_version
        python_management.save()


class NullExtractedInformationHandler(object):
    def handle_extracted_information(self, request, lines_post_processor):
        pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def populate_security_group_instances(apps, schema_editor):
    InstanceSecurityGroup = apps.get_model('openstack', 'InstanceSecurityGroup')

    for isg in InstanceSecurityGroup.objects.all():
        isg.security_group.instances.add(isg.instance)


class Migration(migrations.Migration):

    dependencies = [
        ('openstack', '0017_backup_snapshots_and_restorations'),
    ]

    operations = [
        migrations.AddField(
            model_name='instance',
            name='security_groups',
            field=models.ManyToManyField(related_name='instances', to='openstack.SecurityGroup'),
        ),
        migrations.RunPython(populate_security_group_instances, elidable=True),
        migrations.RemoveField(
            model_name='instancesecuritygroup',
            name='instance',
        ),
        migrations.RemoveField(
            model_name='instancesecuritygroup',
            name='security_group',
        ),
        migrations.DeleteModel(
            name='InstanceSecurityGroup',
        ),
    ]



from django.apps import AppConfig
from django.db.models import signals


class MarketplaceSupportConfig(AppConfig):
    name = 'waldur_mastermind.marketplace_support'
    verbose_name = 'Marketplace supports'

    def ready(self):
        from waldur_mastermind.marketplace.plugins import manager
        from waldur_mastermind.marketplace import models as marketplace_models
        from waldur_mastermind.marketplace import handlers as marketplace_handlers
        from waldur_mastermind.marketplace_support import PLUGIN_NAME
        from waldur_mastermind.support import models as support_models

        from . import handlers, processor

        signals.post_save.connect(
            handlers.create_support_template,
            sender=marketplace_models.Offering,
            dispatch_uid='waldur_mastermind.marketpace_support.create_support_template',
        )

        signals.post_save.connect(
            handlers.update_support_template,
            sender=marketplace_models.Offering,
            dispatch_uid='waldur_mastermind.marketpace_support.update_support_template',
        )

        signals.post_save.connect(
            handlers.change_order_item_state,
            sender=support_models.Offering,
            dispatch_uid='waldur_mastermind.marketpace_support.change_order_item_state',
        )

        signals.pre_delete.connect(
            handlers.terminate_resource,
            sender=support_models.Offering,
            dispatch_uid='waldur_mastermind.marketpace_support.terminate_resource',
        )

        signals.post_save.connect(
            handlers.create_or_update_support_plan,
            sender=marketplace_models.Plan,
            dispatch_uid='waldur_mastermind.marketpace_support.create_or_update_support_plan',
        )

        signals.post_save.connect(
            handlers.change_offering_state,
            sender=support_models.Issue,
            dispatch_uid='waldur_mastermind.marketpace_support.change_offering_state',
        )

        signals.post_save.connect(
            handlers.update_order_item_if_issue_was_complete,
            sender=support_models.Issue,
            dispatch_uid='waldur_mastermind.marketpace_support.update_order_item_if_issue_was_complete',
        )

        signals.post_save.connect(
            handlers.notify_about_request_based_item_creation,
            sender=support_models.Issue,
            dispatch_uid='waldur_mastermind.marketpace_support.notify_about_request_based_item_creation',
        )

        manager.register(PLUGIN_NAME,
                         create_resource_processor=processor.CreateRequestProcessor,
                         update_resource_processor=processor.UpdateRequestProcessor,
                         delete_resource_processor=processor.DeleteRequestProcessor,
                         can_terminate_order_item=True)

        marketplace_handlers.connect_resource_metadata_handlers(support_models.Offering)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-11-20 13:12
from __future__ import unicode_literals

from django.db import migrations


def migrate_resource(apps, schema_editor):
    Resource = apps.get_model('marketplace', 'Resource')
    OrderItem = apps.get_model('marketplace', 'OrderItem')
    ComponentQuota = apps.get_model('marketplace', 'ComponentQuota')
    ComponentUsage = apps.get_model('marketplace', 'ComponentUsage')

    class OrderItemStates(object):
        PENDING = 1
        EXECUTING = 2
        DONE = 3
        ERRED = 4

    class ResourceStates(object):
        CREATING = 1
        OK = 2
        ERRED = 3
        UPDATING = 4
        TERMINATING = 5
        TERMINATED = 6

    StatesMap = {
        OrderItemStates.PENDING: ResourceStates.CREATING,
        OrderItemStates.EXECUTING: ResourceStates.CREATING,
        OrderItemStates.DONE: ResourceStates.OK,
        OrderItemStates.ERRED: ResourceStates.ERRED,
    }

    for item in OrderItem.objects.all():
        item.limits = {q.component.type: q.limit for q in item.quotas.all()}
        resource, _ = Resource.objects.get_or_create(
            object_id=item.object_id,
            content_type=item.content_type,
            plan=item.plan,
            attributes=item.attributes,
            project=item.order.project,
            offering=item.offering,
            limits=item.limits,
            state=StatesMap[item.state],
        )
        item.resource = resource
        item.save()

    for obj in ComponentQuota.objects.all():
        obj.resource = obj.order_item.resource
        obj.save()

    for obj in ComponentUsage.objects.all():
        obj.resource = obj.order_item.resource
        obj.save()


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0045_add_resource'),
    ]

    operations = [
        migrations.RunPython(migrate_resource, elidable=True)
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-03-21 12:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0016_add_states'),
    ]

    operations = [
        migrations.AlterField(
            model_name='comment',
            name='backend_id',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='issue',
            name='backend_id',
            field=models.CharField(blank=True, max_length=255, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='supportuser',
            name='backend_id',
            field=models.CharField(blank=True, max_length=255, null=True, unique=True),
        ),
        migrations.AlterUniqueTogether(
            name='comment',
            unique_together=set([('backend_id', 'issue')]),
        ),
    ]



from django.conf import settings
from django.db import transaction

from waldur_core.logging.loggers import EventLogger, event_logger

from . import models, tasks


class MarketplaceOrderLogger(EventLogger):
    order = models.Order

    class Meta:
        event_types = (
            'marketplace_order_created',
            'marketplace_order_approved',
            'marketplace_order_rejected',
            'marketplace_order_completed',
            'marketplace_order_terminated',
            'marketplace_order_failed',
        )

    @staticmethod
    def get_scopes(event_context):
        order = event_context['order']
        return {order, order.project, order.project.customer}


class MarketplaceResourceLogger(EventLogger):
    resource = models.Resource

    def process(self, level, message_template, event_type='undefined', event_context=None):
        super(MarketplaceResourceLogger, self).process(level, message_template, event_type, event_context)

        if not event_context:
            event_context = {}

        if not settings.WALDUR_MARKETPLACE['NOTIFY_ABOUT_RESOURCE_CHANGE'] or event_type not in (
                'marketplace_resource_create_succeeded',
                'marketplace_resource_create_failed',
                'marketplace_resource_update_succeeded',
                'marketplace_resource_update_failed',
                'marketplace_resource_terminate_succeeded',
                'marketplace_resource_terminate_failed',
                'marketplace_resource_update_limits_succeeded',
                'marketplace_resource_update_limits_failed',
        ):
            return

        if (settings.WALDUR_MARKETPLACE['DISABLE_SENDING_NOTIFICATIONS_ABOUT_RESOURCE_UPDATE'] and
                event_type == 'marketplace_resource_update_succeeded'):
            return

        context = self.compile_context(**event_context)
        resource = event_context['resource']
        transaction.on_commit(lambda: tasks.notify_about_resource_change.delay(event_type, context, resource.uuid))

    class Meta:
        event_types = (
            'marketplace_resource_create_requested',
            'marketplace_resource_create_succeeded',
            'marketplace_resource_create_failed',
            'marketplace_resource_update_requested',
            'marketplace_resource_update_succeeded',
            'marketplace_resource_update_failed',
            'marketplace_resource_terminate_requested',
            'marketplace_resource_terminate_succeeded',
            'marketplace_resource_terminate_failed',
            'marketplace_resource_update_limits_succeeded',
            'marketplace_resource_update_limits_failed',
        )

    @staticmethod
    def get_scopes(event_context):
        resource = event_context['resource']
        return {resource, resource.project, resource.project.customer}


event_logger.register('marketplace_order', MarketplaceOrderLogger)
event_logger.register('marketplace_resource', MarketplaceResourceLogger)


def log_order_created(order):
    event_logger.marketplace_order.info(
        'Marketplace order has been created.',
        event_type='marketplace_order_created',
        event_context={'order': order},
    )


def log_order_approved(order):
    event_logger.marketplace_order.info(
        'Marketplace order has been approved.',
        event_type='marketplace_order_approved',
        event_context={'order': order},
    )


def log_order_rejected(order):
    event_logger.marketplace_order.info(
        'Marketplace order has been rejected.',
        event_type='marketplace_order_rejected',
        event_context={'order': order},
    )


def log_order_completed(order):
    event_logger.marketplace_order.info(
        'Marketplace order has been completed.',
        event_type='marketplace_order_completed',
        event_context={'order': order},
    )


def log_order_terminated(order):
    event_logger.marketplace_order.info(
        'Marketplace order has been terminated.',
        event_type='marketplace_order_terminated',
        event_context={'order': order},
    )


def log_order_failed(order):
    event_logger.marketplace_order.info(
        'Marketplace order has been marked as failed.',
        event_type='marketplace_order_failed',
        event_context={'order': order},
    )


def log_resource_creation_requested(resource):
    event_logger.marketplace_resource.info(
        'Resource {resource_name} creation has been requested.',
        event_type='marketplace_resource_create_requested',
        event_context={'resource': resource},
    )


def log_resource_creation_succeeded(resource):
    event_logger.marketplace_resource.info(
        'Resource {resource_name} has been created.',
        event_type='marketplace_resource_create_succeeded',
        event_context={'resource': resource},
    )


def log_resource_creation_failed(instance):
    event_logger.marketplace_resource.error(
        'Resource {resource_name} creation has failed.',
        event_type='marketplace_resource_create_failed',
        event_context={'resource': instance},
    )


def log_resource_update_requested(resource):
    event_logger.marketplace_resource.info(
        'Resource {resource_name} update has been requested.',
        event_type='marketplace_resource_update_requested',
        event_context={'resource': resource},
    )


def log_resource_update_succeeded(resource):
    event_logger.marketplace_resource.info(
        'Resource {resource_name} has been updated.',
        event_type='marketplace_resource_update_succeeded',
        event_context={'resource': resource},
    )


def log_resource_update_failed(instance):
    event_logger.marketplace_resource.error(
        'Resource {resource_name} update has failed.',
        event_type='marketplace_resource_update_failed',
        event_context={'resource': instance},
    )


def log_resource_terminate_requested(resource):
    event_logger.marketplace_resource.info(
        'Resource {resource_name} deletion been requested.',
        event_type='marketplace_resource_terminate_requested',
        event_context={'resource': resource},
    )


def log_resource_terminate_succeeded(resource):
    event_logger.marketplace_resource.info(
        'Resource {resource_name} has been deleted.',
        event_type='marketplace_resource_terminate_succeeded',
        event_context={'resource': resource},
    )


def log_resource_terminate_failed(instance):
    event_logger.marketplace_resource.error(
        'Resource {resource_name} deletion has failed.',
        event_type='marketplace_resource_terminate_failed',
        event_context={'resource': instance},
    )


def log_resource_limit_update_succeeded(resource):
    event_logger.marketplace_resource.info(
        'Limits of resource {resource_name} have been updated.',
        event_type='marketplace_resource_update_limits_succeeded',
        event_context={'resource': resource},
    )


def log_resource_limit_update_failed(resource):
    event_logger.marketplace_resource.info(
        'Update limits of resource {resource_name} is failed.',
        event_type='marketplace_resource_update_limits_failed',
        event_context={'resource': resource},
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-22 12:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openstack_tenant', '0045_volumeavailabilityzone_available'),
    ]

    operations = [
        migrations.AddField(
            model_name='instanceavailabilityzone',
            name='available',
            field=models.BooleanField(default=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-30 10:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0083_offering_component'),
    ]

    operations = [
        migrations.AddField(
            model_name='serviceprovider',
            name='lead_email',
            field=models.EmailField(blank=True, max_length=254, null=True,
                                    help_text='Email for notification about new request based order items. If this field is set, notifications will be sent.'),
        ),
        migrations.AddField(
            model_name='serviceprovider',
            name='lead_body',
            field=models.TextField(blank=True, help_text='Notification body template. Django template variables can be used.'),
        ),
        migrations.AddField(
            model_name='serviceprovider',
            name='lead_subject',
            field=models.CharField(blank=True, max_length=255, help_text='Notification subject template. Django template variables can be used.'),
        ),
    ]



from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from djangosaml2.backends import Saml2Backend

User = get_user_model()


class WaldurSaml2Backend(Saml2Backend):
    def is_authorized(self, attributes, attribute_mapping):
        email = self.get_attribute_value('email', attributes, attribute_mapping)
        username = self.get_attribute_value('username', attributes, attribute_mapping)
        if email and User.objects.filter(email=email).exclude(username=username).exists():
            raise ValidationError('User with this email already exists')
        return True



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-03 17:02
from __future__ import unicode_literals

from django.db import migrations
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('logging', '0001_squashed_0010'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailhook',
            name='event_groups',
            field=waldur_core.core.fields.JSONField(default=list, verbose_name='List of event groups'),
        ),
        migrations.AlterField(
            model_name='pushhook',
            name='event_groups',
            field=waldur_core.core.fields.JSONField(default=list, verbose_name='List of event groups'),
        ),
        migrations.AlterField(
            model_name='systemnotification',
            name='event_groups',
            field=waldur_core.core.fields.JSONField(default=list, verbose_name='List of event groups'),
        ),
        migrations.AlterField(
            model_name='webhook',
            name='event_groups',
            field=waldur_core.core.fields.JSONField(default=list, verbose_name='List of event groups'),
        ),
    ]



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from waldur_core.core import admin as core_admin
from waldur_core.structure import admin as structure_admin
from django.core.exceptions import ValidationError

from . import executors, models


class JiraPropertyAdmin(core_admin.UpdateOnlyModelAdmin,
                        structure_admin.BackendModelAdmin,
                        admin.ModelAdmin):
    list_display = ('name', 'description', 'settings')
    search_fields = ('name', 'description')


class ProjectTemplateAdmin(JiraPropertyAdmin):
    list_display = ('name', 'description')
    search_fields = ('name', 'description')


class ProjectAdmin(structure_admin.ResourceAdmin):
    actions = ['pull']

    class Pull(core_admin.ExecutorAdminAction):
        executor = executors.ProjectPullExecutor
        short_description = _('Pull')

        def validate(self, service_settings):
            States = models.Project.States
            if service_settings.state not in (States.OK, States.ERRED):
                raise ValidationError(_('Project has to be OK or erred.'))

    pull = Pull()


class IssueAdmin(structure_admin.BackendModelAdmin):
    list_filter = ('project',)
    list_display = ('backend_id', 'type', 'project', 'status', 'reporter_name', 'assignee_name')
    actions = ['pull']

    class Pull(core_admin.ExecutorAdminAction):
        executor = executors.IssueUpdateFromBackendExecutor
        short_description = _('Pull issue')

    pull = Pull()


admin.site.register(models.Priority, JiraPropertyAdmin)
admin.site.register(models.IssueType, JiraPropertyAdmin)
admin.site.register(models.ProjectTemplate, ProjectTemplateAdmin)
admin.site.register(models.Issue, IssueAdmin)
admin.site.register(models.Comment, admin.ModelAdmin)
admin.site.register(models.Project, ProjectAdmin)
admin.site.register(models.JiraService, structure_admin.ServiceAdmin)
admin.site.register(models.JiraServiceProjectLink, structure_admin.ServiceProjectLinkAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-08-30 14:49
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0024_offeringplan'),
    ]

    def migrate_config(apps, schema_editor):
        from waldur_mastermind.common.mixins import UnitPriceMixin

        OfferingTemplate = apps.get_model('support', 'OfferingTemplate')
        OfferingPlan = apps.get_model('support', 'OfferingPlan')

        for template in OfferingTemplate.objects.all():
            OfferingPlan.objects.create(
                template=template,
                name='Default',
                product_code=template.config.get('product_code', ''),
                article_code=template.config.get('article_code', ''),
                unit_price=template.config.get('price', 0),
                unit=template.config.get('unit', UnitPriceMixin.Units.PER_MONTH),
            )

    operations = [
        migrations.RunPython(migrate_config, elidable=True),
    ]



from waldur_ansible.common.extension import AnsibleCommonExtension

from waldur_core.core import WaldurExtension


class JupyterHubManagementExtension(WaldurExtension):
    class Settings:
        WALDUR_JUPYTER_HUB_MANAGEMENT = {
            'JUPYTER_MANAGEMENT_PLAYBOOKS_DIRECTORY': '%swaldur-apps/jupyter_hub_management/' % AnsibleCommonExtension.Settings.WALDUR_ANSIBLE_COMMON['ANSIBLE_LIBRARY'],
        }

    @staticmethod
    def django_app():
        return 'waldur_ansible.jupyter_hub_management'

    @staticmethod
    def is_assembly():
        return True

    @staticmethod
    def rest_urls():
        from .urls import register_in
        return register_in



from __future__ import unicode_literals

from django.contrib.contenttypes.models import ContentType
import django_filters
from django_filters.widgets import BooleanWidget
from rest_framework import filters

from waldur_core.core import serializers as core_serializers, filters as core_filters
from waldur_core.logging import models, utils
from waldur_core.logging.loggers import expand_event_groups


class BaseHookFilter(django_filters.FilterSet):
    author_uuid = django_filters.UUIDFilter(name='user__uuid')
    is_active = django_filters.BooleanFilter(widget=BooleanWidget)
    last_published = django_filters.DateTimeFilter()


class WebHookFilter(BaseHookFilter):
    class Meta(object):
        model = models.WebHook
        fields = ('destination_url', 'content_type')


class EmailHookFilter(BaseHookFilter):
    class Meta(object):
        model = models.EmailHook
        fields = ('email',)


class HookSummaryFilterBackend(core_filters.SummaryFilter):
    def get_queryset_filter(self, queryset):
        if queryset.model == models.WebHook:
            return WebHookFilter
        elif queryset.model == models.EmailHook:
            return EmailHookFilter
        elif queryset.model == models.PushHook:
            return PushHookFilter

        return BaseHookFilter

    def get_base_filter(self):
        return BaseHookFilter


class PushHookFilter(BaseHookFilter):
    class Meta(object):
        model = models.PushHook
        fields = ('type', 'device_id', 'device_manufacturer', 'device_model', 'token')


class EventFilter(django_filters.FilterSet):
    created_from = core_filters.TimestampFilter(name='created', lookup_expr='gte')
    created_to = core_filters.TimestampFilter(name='created', lookup_expr='lt')
    message = django_filters.CharFilter(lookup_expr='icontains')
    o = django_filters.OrderingFilter(fields=('created',))

    class Meta:
        model = models.Event
        fields = []


class EventFilterBackend(filters.BaseFilterBackend):

    def filter_queryset(self, request, queryset, view):
        event_types = request.query_params.getlist('event_type')
        if event_types:
            queryset = queryset.filter(event_type__in=event_types)

        features = request.query_params.getlist('feature')
        if features:
            queryset = queryset.filter(event_type__in=expand_event_groups(features))

        if 'scope' in request.query_params:
            field = core_serializers.GenericRelatedField(related_models=utils.get_loggable_models())
            field._context = {'request': request}
            scope = field.to_internal_value(request.query_params['scope'])

            # Check permissions
            visible = scope._meta.model.get_permitted_objects(request.user)
            if not visible.filter(pk=scope.pk).exists():
                return queryset.none()

            content_type = ContentType.objects.get_for_model(scope._meta.model)
            events = models.Feed.objects.filter(
                content_type=content_type,
                object_id=scope.id,
            ).values_list('event_id', flat=True)
            queryset = queryset.filter(id__in=events)

        elif not request.user.is_staff and not request.user.is_support:
            # If user is not staff nor support, he is allowed to see
            # events related to particular scope only.
            queryset = queryset.none()

        return queryset



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-03 17:43
from __future__ import unicode_literals

from django.db import migrations
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_paypal', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='invoice',
            name='issuer_details',
            field=waldur_core.core.fields.JSONField(blank=True, default=dict, help_text='Stores data about invoice issuer'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='payment_details',
            field=waldur_core.core.fields.JSONField(blank=True, default=dict, help_text='Stores data about customer payment details'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations

from waldur_core.core.utils import month_start


def fill_billing_period(apps, schema_editor):
    ComponentUsage = apps.get_model('marketplace', 'ComponentUsage')
    for item in ComponentUsage.objects.all():
        item.billing_period = month_start(item.date)
        item.save(update_fields=['billing_period'])


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0087_component_usage_billing_period'),
    ]

    operations = [
        migrations.RunPython(fill_billing_period, elidable=True)
    ]



import logging

from celery import chain

from waldur_core.core import executors as core_executors
from waldur_core.core import tasks as core_tasks
from waldur_core.core import utils as core_utils
from waldur_core.structure import executors as structure_executors
from waldur_core.structure import models as structure_models

from . import models, tasks


logger = logging.getLogger(__name__)


class SecurityGroupCreateExecutor(core_executors.CreateExecutor):

    @classmethod
    def get_task_signature(cls, security_group, serialized_security_group, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_security_group, 'create_security_group', state_transition='begin_creating')


class SecurityGroupUpdateExecutor(core_executors.UpdateExecutor):

    @classmethod
    def get_task_signature(cls, security_group, serialized_security_group, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_security_group, 'update_security_group', state_transition='begin_updating')


class SecurityGroupDeleteExecutor(core_executors.DeleteExecutor):

    @classmethod
    def get_task_signature(cls, security_group, serialized_security_group, **kwargs):
        if security_group.backend_id:
            return core_tasks.BackendMethodTask().si(
                serialized_security_group, 'delete_security_group', state_transition='begin_deleting')
        else:
            return core_tasks.StateTransitionTask().si(serialized_security_group, state_transition='begin_deleting')


class PushSecurityGroupRulesExecutor(core_executors.ActionExecutor):

    @classmethod
    def get_task_signature(cls, security_group, serialized_security_group, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_security_group, 'push_security_group_rules', state_transition='begin_updating')


class TenantCreateExecutor(core_executors.CreateExecutor):

    @classmethod
    def get_task_signature(cls, tenant, serialized_tenant, pull_security_groups=True, **kwargs):
        """ Create tenant, add user to it, create internal network, pull quotas """
        # we assume that tenant one network and subnet after creation
        network = tenant.networks.first()
        subnet = network.subnets.first()
        serialized_network = core_utils.serialize_instance(network)
        serialized_subnet = core_utils.serialize_instance(subnet)
        creation_tasks = [
            core_tasks.BackendMethodTask().si(serialized_tenant, 'create_tenant_safe', state_transition='begin_creating'),
            core_tasks.BackendMethodTask().si(serialized_tenant, 'add_admin_user_to_tenant'),
            core_tasks.BackendMethodTask().si(serialized_tenant, 'create_tenant_user'),
            core_tasks.BackendMethodTask().si(serialized_network, 'create_network', state_transition='begin_creating'),
            core_tasks.BackendMethodTask().si(serialized_subnet, 'create_subnet', state_transition='begin_creating'),
        ]
        quotas = tenant.quotas.all()
        quotas = {q.name: int(q.limit) if q.limit.is_integer() else q.limit for q in quotas}
        creation_tasks.append(core_tasks.BackendMethodTask().si(serialized_tenant, 'push_tenant_quotas', quotas))
        # handle security groups
        # XXX: Create default security groups that was connected to SPL earlier.
        for security_group in tenant.security_groups.all():
            creation_tasks.append(SecurityGroupCreateExecutor.as_signature(security_group))

        if pull_security_groups:
            creation_tasks.append(core_tasks.BackendMethodTask().si(serialized_tenant, 'pull_tenant_security_groups'))

        # initialize external network if it defined in service settings
        service_settings = tenant.service_project_link.service.settings
        customer = tenant.service_project_link.project.customer
        external_network_id = service_settings.get_option('external_network_id')

        try:
            customer_openstack = models.CustomerOpenStack.objects.get(
                settings=service_settings,
                customer=customer)
            external_network_id = customer_openstack.external_network_id
        except models.CustomerOpenStack.DoesNotExist:
            pass

        if external_network_id and not kwargs.get('skip_connection_extnet'):
            creation_tasks.append(core_tasks.BackendMethodTask().si(
                serialized_tenant, 'connect_tenant_to_external_network', external_network_id=external_network_id))

        creation_tasks.append(core_tasks.BackendMethodTask().si(serialized_tenant, 'pull_tenant_quotas'))
        return chain(*creation_tasks)

    @classmethod
    def get_success_signature(cls, tenant, serialized_tenant, **kwargs):
        return tasks.TenantCreateSuccessTask().si(serialized_tenant)

    @classmethod
    def get_failure_signature(cls, tenant, serialized_tenant, **kwargs):
        return tasks.TenantCreateErrorTask().s(serialized_tenant)


class TenantImportExecutor(core_executors.ActionExecutor):

    @classmethod
    def get_task_signature(cls, tenant, serialized_tenant, **kwargs):
        tasks = [
            core_tasks.BackendMethodTask().si(serialized_tenant,
                                              'add_admin_user_to_tenant',
                                              state_transition='begin_updating'),
            core_tasks.BackendMethodTask().si(serialized_tenant, 'create_or_update_tenant_user'),
            core_tasks.BackendMethodTask().si(serialized_tenant, 'pull_tenant_quotas'),
            core_tasks.BackendMethodTask().si(serialized_tenant, 'pull_tenant_floating_ips'),
            core_tasks.BackendMethodTask().si(serialized_tenant, 'pull_tenant_security_groups'),
            core_tasks.BackendMethodTask().si(serialized_tenant, 'import_tenant_networks'),
            core_tasks.BackendMethodTask().si(serialized_tenant, 'import_tenant_subnets'),
            core_tasks.BackendMethodTask().si(serialized_tenant, 'detect_external_network'),
        ]

        service_settings = structure_models.ServiceSettings.objects.get(scope=tenant)
        serialized_service_settings = core_utils.serialize_instance(service_settings)
        create_service_settings = structure_executors.ServiceSettingsCreateExecutor.get_task_signature(
            service_settings, serialized_service_settings)

        return chain(*tasks) | create_service_settings

    @classmethod
    def get_success_signature(cls, tenant, serialized_tenant, **kwargs):
        service_settings = structure_models.ServiceSettings.objects.get(scope=tenant)
        serialized_service_settings = core_utils.serialize_instance(service_settings)
        tasks = [
            core_tasks.StateTransitionTask().si(serialized_tenant, state_transition='set_ok'),
            core_tasks.StateTransitionTask().si(serialized_service_settings, state_transition='set_ok')
        ]

        return chain(*tasks)


class TenantUpdateExecutor(core_executors.UpdateExecutor):

    @classmethod
    def get_task_signature(cls, tenant, serialized_tenant, **kwargs):
        updated_fields = kwargs['updated_fields']
        if 'name' in updated_fields or 'description' in updated_fields:
            return core_tasks.BackendMethodTask().si(
                serialized_tenant, 'update_tenant', state_transition='begin_updating')
        else:
            return core_tasks.StateTransitionTask().si(serialized_tenant, state_transition='begin_updating')


class TenantDeleteExecutor(core_executors.DeleteExecutor):

    @classmethod
    def get_task_signature(cls, tenant, serialized_tenant, **kwargs):
        state_transition = core_tasks.StateTransitionTask().si(serialized_tenant, state_transition='begin_deleting')
        if not tenant.backend_id:
            return state_transition

        cleanup_networks = cls.get_networks_cleanup_tasks(serialized_tenant)
        cleanup_instances = cls.get_instances_cleanup_tasks(serialized_tenant)
        cleanup_identities = cls.get_identity_cleanup_tasks(serialized_tenant)

        return chain([state_transition] + cleanup_networks + cleanup_instances + cleanup_identities)

    @classmethod
    def get_networks_cleanup_tasks(cls, serialized_tenant):
        return [
            core_tasks.BackendMethodTask().si(
                serialized_tenant, backend_method='delete_tenant_floating_ips',
            ),
            core_tasks.BackendMethodTask().si(
                serialized_tenant, backend_method='delete_tenant_ports',
            ),
            core_tasks.BackendMethodTask().si(
                serialized_tenant, backend_method='delete_tenant_routers',
            ),
            core_tasks.BackendMethodTask().si(
                serialized_tenant, backend_method='delete_tenant_networks',
            ),
        ]

    @classmethod
    def get_instances_cleanup_tasks(cls, serialized_tenant):
        return [
            core_tasks.BackendMethodTask().si(
                serialized_tenant, backend_method='delete_tenant_security_groups',
            ),
            core_tasks.BackendMethodTask().si(
                serialized_tenant, backend_method='delete_tenant_snapshots',
            ),
            core_tasks.PollBackendCheckTask().si(
                serialized_tenant,
                backend_check_method='are_all_tenant_snapshots_deleted'
            ),
            core_tasks.BackendMethodTask().si(
                serialized_tenant, backend_method='delete_tenant_instances',
            ),
            core_tasks.PollBackendCheckTask().si(
                serialized_tenant,
                backend_check_method='are_all_tenant_instances_deleted'
            ),
            core_tasks.BackendMethodTask().si(
                serialized_tenant, backend_method='delete_tenant_volumes',
            ),
            core_tasks.PollBackendCheckTask().si(
                serialized_tenant,
                backend_check_method='are_all_tenant_volumes_deleted'
            ),
        ]

    @classmethod
    def get_identity_cleanup_tasks(cls, serialized_tenant):
        return [
            core_tasks.BackendMethodTask().si(
                serialized_tenant, backend_method='delete_tenant_user',
            ),
            core_tasks.BackendMethodTask().si(
                serialized_tenant, backend_method='delete_tenant',
            ),
        ]


class TenantAllocateFloatingIPExecutor(core_executors.ActionExecutor):

    @classmethod
    def get_task_signature(cls, tenant, serialized_tenant, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_tenant, 'allocate_floating_ip_address', state_transition='begin_updating')


class FloatingIPCreateExecutor(core_executors.CreateExecutor):

    @classmethod
    def get_task_signature(cls, floating_ip, serialized_floating_ip, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_floating_ip, 'create_floating_ip', state_transition='begin_creating')


class FloatingIPDeleteExecutor(core_executors.DeleteExecutor):

    @classmethod
    def get_task_signature(cls, floating_ip, serialized_floating_ip, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_floating_ip, 'delete_floating_ip', state_transition='begin_deleting')


class FloatingIPPullExecutor(core_executors.ActionExecutor):

    @classmethod
    def get_task_signature(cls, floating_ip, serialized_floating_ip, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_floating_ip, 'pull_floating_ip', state_transition='begin_updating')


class TenantPullFloatingIPsExecutor(core_executors.ActionExecutor):

    @classmethod
    def get_task_signature(cls, tenant, serialized_tenant, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_tenant, 'pull_tenant_floating_ips', state_transition='begin_updating')


class TenantPushQuotasExecutor(core_executors.ActionExecutor):

    @classmethod
    def get_task_signature(cls, tenant, serialized_tenant, quotas=None, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_tenant, 'push_tenant_quotas', quotas, state_transition='begin_updating')


class TenantPullQuotasExecutor(core_executors.ActionExecutor):

    @classmethod
    def get_task_signature(cls, tenant, serialized_tenant, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_tenant, 'pull_tenant_quotas', state_transition='begin_updating')


class TenantPullExecutor(core_executors.ActionExecutor):

    @classmethod
    def get_task_signature(cls, tenant, serialized_tenant, **kwargs):
        service_settings = structure_models.ServiceSettings.objects.get(scope=tenant)
        serialized_settings = core_utils.serialize_instance(service_settings)
        return chain(
            core_tasks.BackendMethodTask().si(
                serialized_tenant, 'pull_tenant', state_transition='begin_updating'),
            core_tasks.BackendMethodTask().si(serialized_tenant, 'pull_tenant_quotas'),
            core_tasks.BackendMethodTask().si(serialized_tenant, 'pull_tenant_floating_ips'),
            core_tasks.BackendMethodTask().si(serialized_tenant, 'pull_tenant_security_groups'),
            core_tasks.IndependentBackendMethodTask().si(serialized_settings, 'pull_images'),
            core_tasks.IndependentBackendMethodTask().si(serialized_settings, 'pull_flavors'),
        )


class TenantPullSecurityGroupsExecutor(core_executors.ActionExecutor):

    @classmethod
    def get_task_signature(cls, tenant, serialized_tenant, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_tenant, 'pull_tenant_security_groups',
            state_transition='begin_updating')


class TenantDetectExternalNetworkExecutor(core_executors.ActionExecutor):

    @classmethod
    def get_task_signature(cls, tenant, serialized_tenant, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_tenant, 'detect_external_network',
            state_transition='begin_updating')


class TenantChangeUserPasswordExecutor(core_executors.ActionExecutor):

    @classmethod
    def get_task_signature(cls, tenant, serialized_tenant, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_tenant, 'change_tenant_user_password',
            state_transition='begin_updating')


class NetworkCreateExecutor(core_executors.CreateExecutor):

    @classmethod
    def get_task_signature(cls, network, serialized_network, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_network, 'create_network', state_transition='begin_creating')


class NetworkUpdateExecutor(core_executors.UpdateExecutor):

    @classmethod
    def get_task_signature(cls, network, serialized_network, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_network, 'update_network', state_transition='begin_updating')


class NetworkDeleteExecutor(core_executors.DeleteExecutor):

    @classmethod
    def get_task_signature(cls, network, serialized_network, **kwargs):
        if network.backend_id:
            return core_tasks.BackendMethodTask().si(
                serialized_network, 'delete_network', state_transition='begin_deleting')
        else:
            return core_tasks.StateTransitionTask().si(serialized_network, state_transition='begin_deleting')


class NetworkPullExecutor(core_executors.ActionExecutor):
    action = 'pull'

    @classmethod
    def get_task_signature(cls, network, serialized_network, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_network, 'pull_network', state_transition='begin_updating')


class SubNetCreateExecutor(core_executors.CreateExecutor):

    @classmethod
    def get_task_signature(cls, subnet, serialized_subnet, **kwargs):
        return core_tasks.BackendMethodTask().si(serialized_subnet, 'create_subnet', state_transition='begin_creating')


class SubNetUpdateExecutor(core_executors.UpdateExecutor):

    @classmethod
    def get_task_signature(cls, subnet, serialized_subnet, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_subnet, 'update_subnet', state_transition='begin_updating')


class SubNetDeleteExecutor(core_executors.DeleteExecutor):

    @classmethod
    def get_task_signature(cls, subnet, serialized_subnet, **kwargs):
        if subnet.backend_id:
            return core_tasks.BackendMethodTask().si(
                serialized_subnet, 'delete_subnet', state_transition='begin_deleting')
        else:
            return core_tasks.StateTransitionTask().si(serialized_subnet, state_transition='begin_deleting')


class SubNetPullExecutor(core_executors.ActionExecutor):
    action = 'pull'

    @classmethod
    def get_task_signature(cls, subnet, serialized_subnet, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_subnet, 'pull_subnet', state_transition='begin_updating')


class OpenStackCleanupExecutor(structure_executors.BaseCleanupExecutor):
    executors = (
        (models.SecurityGroup, SecurityGroupDeleteExecutor),
        (models.FloatingIP, FloatingIPDeleteExecutor),
        (models.SubNet, SubNetDeleteExecutor),
        (models.Network, NetworkDeleteExecutor),
        (models.Tenant, TenantDeleteExecutor),
    )



import json

from django import forms
from django.contrib import admin
from django.contrib.auth.models import Group
from django.db import transaction
from django.shortcuts import redirect
from django.template.defaultfilters import filesizeformat
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
import six

from waldur_core.core.admin import (
    format_json_field, ReadOnlyAdminMixin,
    ExtraActionsMixin, UpdateOnlyModelAdmin
)
from waldur_core.core.utils import serialize_instance
from waldur_core.logging import models, tasks
from waldur_core.logging.loggers import get_valid_events, get_event_groups


class JSONMultipleChoiceField(forms.MultipleChoiceField):

    def prepare_value(self, value):
        if isinstance(value, six.string_types):
            return json.loads(value)
        return value


class BaseHookForm(forms.ModelForm):
    event_types = JSONMultipleChoiceField(
        choices=lambda: [(e, e) for e in get_valid_events()],
        widget=forms.SelectMultiple(attrs={'size': '30'}),
        required=False,
    )

    event_groups = JSONMultipleChoiceField(
        choices=lambda: [(g, g) for g in get_event_groups()],
        widget=forms.SelectMultiple(attrs={'size': '30'}),
        required=False,
    )


class SystemNotificationForm(BaseHookForm):
    roles = JSONMultipleChoiceField(
        choices=[(g, g) for g in models.SystemNotification.get_valid_roles()],
        widget=forms.SelectMultiple(attrs={'size': '30'}),
        required=True,
    )

    class Meta:
        model = models.SystemNotification
        exclude = 'uuid',

    def __init__(self, *args, **kwargs):
        super(SystemNotificationForm, self).__init__(*args, **kwargs)
        self.fields['hook_content_type'].queryset = models.BaseHook.get_all_content_types()


class SystemNotificationAdmin(admin.ModelAdmin):
    form = SystemNotificationForm
    list_display = ('name', 'hook_content_type', 'roles')


class EventAdmin(ReadOnlyAdminMixin, admin.ModelAdmin):
    list_display = ('uuid', 'event_type', 'message', 'created')
    readonly_fields = ('event_type', 'message', 'format_context')
    exclude = ('context',)
    list_filter = ('event_type', 'created')
    search_fields = ('message',)
    ordering = ('-created',)

    def format_context(self, obj):
        return format_json_field(obj.context)

    format_context.allow_tags = True
    format_context.short_description = _('Details')


class BaseHookAdmin(admin.ModelAdmin):
    form = BaseHookForm
    list_display = ('uuid', 'user', 'is_active', 'event_types', 'event_groups')


class WebHookAdmin(BaseHookAdmin):
    list_display = BaseHookAdmin.list_display + ('destination_url',)


class EmailHookAdmin(BaseHookAdmin):
    list_display = BaseHookAdmin.list_display + ('email',)


class PushHookAdmin(BaseHookAdmin):
    list_display = BaseHookAdmin.list_display + ('type', 'device_id')


class ReportAdmin(UpdateOnlyModelAdmin, ExtraActionsMixin, admin.ModelAdmin):
    list_display = ('created', 'state', 'get_filesize')
    readonly_fields = ('state', 'file', 'get_filesize', 'error_message')
    exclude = ('file_size',)

    def get_filesize(self, obj):
        if obj.file_size:
            return filesizeformat(obj.file_size)
        else:
            return 'N/A'

    get_filesize.short_description = 'File size'

    def get_extra_actions(self):
        return [self.create_report]

    def create_report(self, request):
        with transaction.atomic():
            report = models.Report.objects.create()
            serialized_report = serialize_instance(report)
            transaction.on_commit(lambda: tasks.create_report.delay(serialized_report))
        message = _('Report creation has been scheduled')
        self.message_user(request, message)
        return redirect(reverse('admin:logging_report_changelist'))

    create_report.short_description = _('Create report')


# This hack is needed because core admin is imported several times.
if admin.site.is_registered(Group):
    admin.site.unregister(Group)

admin.site.register(models.SystemNotification, SystemNotificationAdmin)
admin.site.register(models.WebHook, WebHookAdmin)
admin.site.register(models.EmailHook, EmailHookAdmin)
admin.site.register(models.PushHook, PushHookAdmin)
admin.site.register(models.Report, ReportAdmin)
admin.site.register(models.Event, EventAdmin)



from waldur_core.core import executors as core_executors, tasks as core_tasks
from waldur_core.structure import executors as structure_executors

from . import models


class AllocationCreateExecutor(core_executors.CreateExecutor):

    @classmethod
    def get_task_signature(cls, volume, serialized_allocation, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_allocation,
            'create_allocation',
            state_transition='begin_creating'
        )


class AllocationUpdateExecutor(core_executors.UpdateExecutor):

    @classmethod
    def get_task_signature(cls, volume, serialized_volume, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_volume,
            'set_resource_limits',
            state_transition='begin_updating'
        )


class AllocationPullExecutor(core_executors.ActionExecutor):
    action = 'Pull'

    @classmethod
    def get_task_signature(cls, volume, serialized_volume, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_volume, 'pull_allocation',
            state_transition='begin_updating')


class AllocationDeleteExecutor(core_executors.DeleteExecutor):

    @classmethod
    def get_task_signature(cls, volume, serialized_allocation, **kwargs):
        return core_tasks.BackendMethodTask().si(
            serialized_allocation,
            'delete_allocation',
            state_transition='begin_deleting'
        )


class SlurmCleanupExecutor(structure_executors.BaseCleanupExecutor):
    executors = (
        (models.Allocation, AllocationDeleteExecutor),
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('openstack_tenant', '0005_resources_actions'),
    ]

    operations = [
        migrations.AlterField(
            model_name='instance',
            name='action_details',
            field=waldur_core.core.fields.JSONField(default={}),
        ),
        migrations.AlterField(
            model_name='snapshot',
            name='action_details',
            field=waldur_core.core.fields.JSONField(default={}),
        ),
        migrations.AlterField(
            model_name='volume',
            name='action_details',
            field=waldur_core.core.fields.JSONField(default={}),
        ),
    ]



from __future__ import unicode_literals

import logging

from django_filters.rest_framework import DjangoFilterBackend

from waldur_core.core import validators as core_validators
from waldur_core.core import views as core_views
from waldur_core.structure import filters as structure_filters
from waldur_core.structure import views as structure_views

from . import models, serializers, filters, executors

logger = logging.getLogger(__name__)


class ServiceViewSet(structure_views.BaseServiceViewSet):
    queryset = models.RancherService.objects.all()
    serializer_class = serializers.ServiceSerializer


class ServiceProjectLinkViewSet(structure_views.BaseServiceProjectLinkViewSet):
    queryset = models.RancherServiceProjectLink.objects.all()
    serializer_class = serializers.ServiceProjectLinkSerializer
    filter_class = filters.ServiceProjectLinkFilter


class ClusterViewSet(structure_views.ImportableResourceViewSet):
    queryset = models.Cluster.objects.all()
    serializer_class = serializers.ClusterSerializer
    filter_class = filters.ClusterFilter
    create_executor = executors.ClusterCreateExecutor
    delete_executor = executors.ClusterDeleteExecutor
    update_executor = executors.ClusterUpdateExecutor
    update_validators = partial_update_validators = [
        core_validators.StateValidator(models.Cluster.States.OK),
    ]
    importable_resources_backend_method = 'get_clusters_for_import'
    importable_resources_serializer_class = serializers.ClusterImportableSerializer
    import_resource_serializer_class = serializers.ClusterImportSerializer


class NodeViewSet(core_views.ActionsViewSet):
    queryset = models.Node.objects.all()
    filter_backends = (structure_filters.GenericRoleFilter, DjangoFilterBackend)
    serializer_class = serializers.NodeSerializer
    filter_class = filters.NodeFilter
    lookup_field = 'uuid'
    disabled_actions = ['update', 'partial_update']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations

from waldur_mastermind.marketplace_openstack import STORAGE_TYPE, RAM_TYPE, CORES_TYPE, PACKAGE_TYPE


def create_category_components(apps, schema_editor):
    CATEGORY_TITLE = 'Private clouds'

    Category = apps.get_model('marketplace', 'Category')
    CategoryComponent = apps.get_model('marketplace', 'CategoryComponent')
    OfferingComponent = apps.get_model('marketplace', 'OfferingComponent')

    try:
        vpc_category = Category.objects.get(title=CATEGORY_TITLE)
    except Category.DoesNotExist:
        return

    storage_gb_cc, _ = CategoryComponent.objects.get_or_create(
        category=vpc_category,
        type=STORAGE_TYPE,
        name='Storage',
        measured_unit='GB'
    )

    ram_gb_cc, _ = CategoryComponent.objects.get_or_create(
        category=vpc_category,
        type=RAM_TYPE,
        name='RAM',
        measured_unit='GB'
    )

    cores_cc, _ = CategoryComponent.objects.get_or_create(
        category=vpc_category,
        type=CORES_TYPE,
        name='Cores',
        measured_unit='cores'
    )

    components = OfferingComponent.objects.filter(offering__type=PACKAGE_TYPE, parent=None)

    components.filter(type=STORAGE_TYPE).update(parent=storage_gb_cc)
    components.filter(type=RAM_TYPE).update(parent=ram_gb_cc)
    components.filter(type=CORES_TYPE).update(parent=cores_cc)


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0082_orderitem_activated'),
    ]

    operations = [
        migrations.RunPython(create_category_components, elidable=True),
    ]



import base64
import logging

from django.conf import settings
from django.contrib import auth
from django.core.exceptions import ValidationError
from django.http import HttpResponseRedirect, JsonResponse
from django.utils.translation import ugettext_lazy as _
from djangosaml2.cache import OutstandingQueriesCache, IdentityCache, StateCache
from djangosaml2.conf import get_config
from djangosaml2.signals import post_authenticated
from djangosaml2.utils import get_custom_setting, get_location
from djangosaml2.views import _set_subject_id, _get_subject_id
from rest_framework.authtoken.models import Token
from rest_framework.generics import ListAPIView
from rest_framework.views import APIView
from saml2 import BINDING_HTTP_POST, BINDING_HTTP_REDIRECT
from saml2.client import Saml2Client
from saml2.response import StatusRequestDenied
from saml2.xmldsig import SIG_RSA_SHA1, DIGEST_SHA1

from waldur_core.core.views import (
    login_failed, login_completed, logout_failed, logout_completed, RefreshTokenMixin,
    validate_authentication_method)

from . import filters, models, serializers, utils
from .log import event_logger


logger = logging.getLogger(__name__)

validate_saml2 = validate_authentication_method('SAML2')


class BaseSaml2View(APIView):
    throttle_classes = ()
    permission_classes = ()
    authentication_classes = ()


class Saml2LoginView(BaseSaml2View):
    """
    SAML Authorization endpoint

    This view receives authorization requests from users and
    redirects them to corresponding IdP authorization page.
    The "metadata" has to be set in SAML_CONFIG in settings.py
    """
    serializer_class = serializers.Saml2LoginSerializer

    @validate_saml2
    def post(self, request):
        if not self.request.user.is_anonymous:
            error_message = _('This endpoint is for anonymous users only.')
            return JsonResponse({'error_message': error_message}, status=400)

        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        idp = serializer.validated_data.get('idp')

        conf = get_config(request=request)

        # ensure our selected binding is supported by the IDP
        supported_bindings = utils.get_idp_sso_supported_bindings(idp, config=conf)
        default_binding = settings.WALDUR_AUTH_SAML2.get('DEFAULT_BINDING')

        if default_binding in supported_bindings:
            binding = default_binding
        elif BINDING_HTTP_POST in supported_bindings:
            binding = BINDING_HTTP_POST
        elif BINDING_HTTP_REDIRECT in supported_bindings:
            binding = BINDING_HTTP_REDIRECT
        else:
            error_message = _('Identity provider does not support available bindings.')
            return JsonResponse({'error_message': error_message}, status=400)

        client = Saml2Client(conf)

        kwargs = {}
        sign_requests = getattr(conf, '_sp_authn_requests_signed', False)
        if sign_requests:
            signature_algorithm = settings.WALDUR_AUTH_SAML2.get('signature_algorithm') or SIG_RSA_SHA1
            digest_algorithm = settings.WALDUR_AUTH_SAML2.get('digest_algorithm') or DIGEST_SHA1

            kwargs['sign'] = True
            kwargs['sigalg'] = signature_algorithm
            kwargs['sign_alg'] = signature_algorithm
            kwargs['digest_alg'] = digest_algorithm

        nameid_format = settings.WALDUR_AUTH_SAML2.get('nameid_format')
        if nameid_format or nameid_format == "":  # "" is a valid setting in pysaml2
            kwargs['nameid_format'] = nameid_format

        if binding == BINDING_HTTP_REDIRECT:
            session_id, result = client.prepare_for_authenticate(
                entityid=idp, binding=binding, **kwargs)

            data = {
                'binding': 'redirect',
                'url': get_location(result),
            }
        elif binding == BINDING_HTTP_POST:
            try:
                location = client.sso_location(idp, binding)
            except TypeError:
                error_message = _('Invalid identity provider specified.')
                return JsonResponse({'error_message': error_message}, status=400)

            session_id, request_xml = client.create_authn_request(location, binding=binding, **kwargs)
            data = {
                'binding': 'post',
                'url': location,
                'request': base64.b64encode(request_xml.encode('UTF-8')),
            }

        # save session_id
        oq_cache = OutstandingQueriesCache(request.session)
        oq_cache.set(session_id, '')

        return JsonResponse(data)


class Saml2LoginCompleteView(RefreshTokenMixin, BaseSaml2View):
    """
    SAML Authorization Response endpoint

    The IdP will send its response to this view, which
    will process it with pysaml2 help and log the user
    in using the custom Authorization backend
    djangosaml2.backends.Saml2Backend that should be
    enabled in the settings.py
    """
    serializer_class = serializers.Saml2LoginCompleteSerializer

    @validate_saml2
    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)

        attribute_mapping = get_custom_setting(
            'SAML_ATTRIBUTE_MAPPING', {'uid': ('username', )})
        create_unknown_user = get_custom_setting(
            'SAML_CREATE_UNKNOWN_USER', True)

        conf = get_config(request=request)
        client = Saml2Client(conf, identity_cache=IdentityCache(request.session))

        oq_cache = OutstandingQueriesCache(request.session)
        outstanding_queries = oq_cache.outstanding_queries()

        xmlstr = serializer.validated_data['SAMLResponse']

        # process the authentication response
        try:
            response = client.parse_authn_request_response(xmlstr, BINDING_HTTP_POST, outstanding_queries)
        except Exception as e:
            if isinstance(e, StatusRequestDenied):
                return login_failed(_('Authentication request has been denied by identity provider. '
                                      'Please check your credentials.'))
            logger.error('SAML response parsing failed %s' % e)
            return login_failed(_('SAML2 response has errors.'))

        if response is None:
            logger.error('SAML response is None')
            return login_failed(_('SAML response has errors. Please check the logs'))

        if response.assertion is None:
            logger.error('SAML response assertion is None')
            return login_failed(_('SAML response has errors. Please check the logs'))

        session_id = response.session_id()
        oq_cache.delete(session_id)

        # authenticate the remote user
        session_info = response.session_info()

        if callable(attribute_mapping):
            attribute_mapping = attribute_mapping()
        if callable(create_unknown_user):
            create_unknown_user = create_unknown_user()

        try:
            user = auth.authenticate(
                session_info=session_info,
                attribute_mapping=attribute_mapping,
                create_unknown_user=create_unknown_user,
            )
        except ValidationError as e:
            return login_failed(e.message)
        if user is None:
            return login_failed(_('SAML2 authentication failed.'))

        registration_method = settings.WALDUR_AUTH_SAML2.get('name', 'saml2')
        if user.registration_method != registration_method:
            user.registration_method = registration_method
            user.save(update_fields=['registration_method'])

        # required for validating SAML2 logout requests
        auth.login(request, user)
        _set_subject_id(request.session, session_info['name_id'])
        logger.debug('User %s authenticated via SSO.', user)

        logger.debug('Sending the post_authenticated signal')
        post_authenticated.send_robust(sender=user, session_info=session_info)
        token = self.refresh_token(user)

        logger.info('Authenticated with SAML token. Returning token for successful login of user %s', user)
        event_logger.saml2_auth.info(
            'User {user_username} with full name {user_full_name} logged in successfully with SAML2.',
            event_type='auth_logged_in_with_saml2', event_context={'user': user}
        )
        return login_completed(token.key, 'saml2')


class Saml2LogoutView(BaseSaml2View):
    """
    SAML Logout endpoint

    This view redirects users to corresponding IdP page for the logout.
    """

    @validate_saml2
    def get(self, request):
        state = StateCache(request.session)
        conf = get_config(request=request)

        client = Saml2Client(conf, state_cache=state, identity_cache=IdentityCache(request.session))
        subject_id = _get_subject_id(request.session)
        if subject_id is None:
            return logout_failed(_('You cannot be logged out.'))

        try:
            result = client.global_logout(subject_id)
        except KeyError:
            return logout_failed(_('You are not logged in any IdP/AA.'))

        state.sync()
        if not result:
            return logout_failed(_('You are not logged in any IdP/AA.'))

        # Logout is supported only from 1 IdP
        binding, http_info = result.values()[0]
        return HttpResponseRedirect(get_location(http_info))


class Saml2LogoutCompleteView(BaseSaml2View):
    """
    SAML Logout Response endpoint

    The IdP will send its response to this view, which
    will logout the user and remove authorization token.
    """

    serializer_class = serializers.Saml2LogoutCompleteSerializer

    @validate_saml2
    def get(self, request):
        """
        For IdPs which send GET requests
        """
        serializer = self.serializer_class(data=request.GET)
        serializer.is_valid(raise_exception=True)
        return self.logout(request, serializer.validated_data, BINDING_HTTP_REDIRECT)

    @validate_saml2
    def post(self, request):
        """
        For IdPs which send POST requests
        """
        serializer = self.serializer_class(data=request.POST)
        serializer.is_valid(raise_exception=True)
        return self.logout(request, serializer.validated_data, BINDING_HTTP_POST)

    def logout(self, request, data, binding):
        conf = get_config(request=request)

        state = StateCache(request.session)
        client = Saml2Client(conf, state_cache=state,
                             identity_cache=IdentityCache(request.session))

        if 'SAMLResponse' in data:
            # Logout started by us
            client.parse_logout_request_response(data['SAMLResponse'], binding)
            http_response = logout_completed()
        else:
            # Logout started by IdP
            subject_id = _get_subject_id(request.session)
            if subject_id is None:
                http_response = logout_completed()
            else:
                http_info = client.handle_logout_request(data['SAMLRequest'], subject_id, binding,
                                                         relay_state=data.get('RelayState', ''))
                http_response = HttpResponseRedirect(get_location(http_info))

        state.sync()
        user = request.user
        if user.is_anonymous:
            return http_response
        Token.objects.get(user=user).delete()
        auth.logout(request)
        event_logger.saml2_auth.info(
            'User {user_username} with full name {user_full_name} logged out successfully with SAML2.',
            event_type='auth_logged_out_with_saml2', event_context={'user': user}
        )
        return http_response


class Saml2ProviderView(ListAPIView):
    throttle_classes = ()
    permission_classes = ()
    serializer_class = serializers.Saml2ProviderSerializer
    queryset = models.IdentityProvider.objects.all()
    filter_class = filters.IdentityProviderFilter



from django.core.management.base import BaseCommand

from waldur_azure.models import Image
from waldur_core.structure.models import ServiceSettings


class Command(BaseCommand):
    help = 'Import Azure image'

    def add_arguments(self, parser):
        parser.add_argument('--sku')
        parser.add_argument('--publisher')
        parser.add_argument('--offer')

    def handle(self, *args, **options):
        for settings in ServiceSettings.objects.filter(type='Azure'):
            Image.objects.update_or_create(
                settings=settings,
                name=options['offer'],
                sku=options['sku'],
                publisher=options['publisher'],
                version='latest',
            )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-08-21 08:45
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0089_make_billing_period_non_nullable'),
    ]

    operations = [
        migrations.AddField(
            model_name='resource',
            name='current_usages',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict),
        ),
    ]



import logging

import requests

from .exceptions import RancherException

logger = logging.getLogger(__name__)


class RancherClient(object):
    """
    Rancher API client.
    See also: https://rancher.com/docs/rancher/v2.x/en/api/
    """
    def __init__(self, host, verify_ssl=True):
        """
        Initialize client with connection options.

        :param host: Rancher server IP address
        :type host: string
        :param verify_ssl: verify SSL certificates for HTTPS requests
        :type verify_ssl: bool
        """
        self._host = host
        self._base_url = '{0}/v3'.format(self._host)
        self._session = requests.Session()
        self._session.verify = verify_ssl

    def _request(self, method, endpoint, json=None, **kwargs):
        url = '%s/%s' % (self._base_url, endpoint)

        try:
            response = self._session.request(method, url, json=json, **kwargs)
        except requests.RequestException as e:
            raise RancherException(e)

        status_code = response.status_code
        if status_code in (requests.codes.ok,
                           requests.codes.created,
                           requests.codes.accepted,
                           requests.codes.no_content):
            if response.content:
                data = response.json()
                if isinstance(data, dict) and 'value' in data:
                    return data['value']
                return data
        else:
            raise RancherException(response.content)

    def _get(self, endpoint, **kwargs):
        return self._request('get', endpoint, **kwargs)

    def _post(self, endpoint, **kwargs):
        return self._request('post', endpoint, **kwargs)

    def _patch(self, endpoint, **kwargs):
        return self._request('patch', endpoint, **kwargs)

    def _delete(self, endpoint, **kwargs):
        return self._request('delete', endpoint, **kwargs)

    def _put(self, endpoint, **kwargs):
        return self._request('put', endpoint, **kwargs)

    def login(self, access_key, secret_key):
        """
        Login to Rancher server using access_key and secret_key.

        :param access_key: access_key to connect
        :type access_key: string
        :param secret_key: secret_key to connect
        :type secret_key: string
        :raises Unauthorized: raised if credentials are invalid.
        """
        self._post('', auth=(access_key, secret_key))
        self._session.auth = (access_key, secret_key)
        logger.debug('Successfully logged in as {0}'.format(access_key))

    def list_clusters(self):
        return self._get('clusters')['data']

    def get_cluster(self, cluster_id):
        return self._get('clusters/{0}'.format(cluster_id))

    def create_cluster(self, cluster_name):
        return self._post('clusters', json={"name": cluster_name, "rancherKubernetesEngineConfig": {}})

    def delete_cluster(self, cluster_id):
        return self._delete('clusters/{0}'.format(cluster_id))

    def list_cluster_registration_tokens(self):
        return self._get('clusterregistrationtokens', params={'limit': -1})['data']

    def create_cluster_registration_token(self, cluster_id):
        return self._post('clusterregistrationtoken',
                          json={"type": "clusterRegistrationToken", "clusterId": cluster_id})

    def get_node_command(self, cluster_id):
        cluster_list = self.list_cluster_registration_tokens()
        cluster = filter(lambda x: x['clusterId'] == cluster_id, cluster_list)
        if cluster:
            node_command = cluster[0]['nodeCommand']
            node_command += ' --etcd --controlplane --worker'
            return node_command

    def update_cluster(self, cluster_id, new_params):
        return self._put('clusters/{0}'.format(cluster_id), json=new_params)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-06-13 06:46
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0003_attribute_section'),
    ]

    operations = [
        migrations.RenameField(
            model_name='offering',
            old_name='features',
            new_name='attributes',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-07-03 12:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_vmware', '0009_customer_network'),
    ]

    operations = [
        migrations.AddField(
            model_name='virtualmachine',
            name='networks',
            field=models.ManyToManyField(blank=True, to='waldur_vmware.Network'),
        ),
    ]



from __future__ import unicode_literals

import django_filters

from waldur_core.core import filters as core_filters
from waldur_core.structure import filters as structure_filters

from . import models


class ServiceProjectLinkFilter(structure_filters.BaseServiceProjectLinkFilter):
    service = core_filters.URLFilter(view_name='vmware-detail', name='service__uuid')

    class Meta(structure_filters.BaseServiceProjectLinkFilter.Meta):
        model = models.VMwareServiceProjectLink


class VirtualMachineFilter(structure_filters.BaseResourceFilter):
    class Meta(structure_filters.BaseResourceFilter.Meta):
        model = models.VirtualMachine
        fields = structure_filters.BaseResourceFilter.Meta.fields + ('runtime_state',)


class PortFilter(structure_filters.BaseResourceFilter):
    class Meta(structure_filters.BaseResourceFilter.Meta):
        model = models.Port

    vm = core_filters.URLFilter(view_name='vmware-virtual-machine-detail', name='vm__uuid')
    vm_uuid = django_filters.UUIDFilter(name='vm__uuid')

    network = core_filters.URLFilter(view_name='vmware-network-detail', name='network__uuid')
    network_uuid = django_filters.UUIDFilter(name='network__uuid')


class DiskFilter(structure_filters.BaseResourceFilter):
    class Meta(structure_filters.BaseResourceFilter.Meta):
        model = models.Disk

    vm = core_filters.URLFilter(view_name='vmware-virtual-machine-detail', name='vm__uuid')
    vm_uuid = django_filters.UUIDFilter(name='vm__uuid')
    ORDERING_FIELDS = structure_filters.BaseResourceFilter.ORDERING_FIELDS + (
        ('size', 'size'),
    )


class TemplateFilter(structure_filters.ServicePropertySettingsFilter):
    class Meta(structure_filters.ServicePropertySettingsFilter.Meta):
        model = models.Template


class ClusterFilter(structure_filters.ServicePropertySettingsFilter):
    customer_uuid = django_filters.UUIDFilter(method='filter_customer', label='Customer UUID')

    def filter_customer(self, queryset, name, value):
        return queryset.filter(customercluster__customer__uuid=value)

    class Meta(structure_filters.ServicePropertySettingsFilter.Meta):
        model = models.Cluster


class NetworkFilter(structure_filters.ServicePropertySettingsFilter):
    customer_uuid = django_filters.UUIDFilter(method='filter_customer', label='Customer UUID')
    customer_pair_uuid = django_filters.UUIDFilter(method='filter_customer_pair', label='Customer UUID')

    def filter_customer(self, queryset, name, value):
        return queryset.filter(customernetwork__customer__uuid=value)

    def filter_customer_pair(self, queryset, name, value):
        return queryset.filter(customernetworkpair__customer__uuid=value)

    class Meta(structure_filters.ServicePropertySettingsFilter.Meta):
        model = models.Network


class DatastoreFilter(structure_filters.ServicePropertySettingsFilter):
    customer_uuid = django_filters.UUIDFilter(method='filter_customer', label='Customer UUID')

    def filter_customer(self, queryset, name, value):
        return queryset.filter(customerdatastore__customer__uuid=value)

    class Meta(structure_filters.ServicePropertySettingsFilter.Meta):
        model = models.Datastore


class FolderFilter(structure_filters.ServicePropertySettingsFilter):
    customer_uuid = django_filters.UUIDFilter(method='filter_customer', label='Customer UUID')

    def filter_customer(self, queryset, name, value):
        return queryset.filter(customerfolder__customer__uuid=value)

    class Meta(structure_filters.ServicePropertySettingsFilter.Meta):
        model = models.Folder



from waldur_core.quotas import fields as quota_fields
from waldur_core.structure import models as structure_models

from . import models


TENANT_QUOTAS = (
    ('vpc_cpu_count', 'vcpu'),
    ('vpc_ram_size', 'ram'),
    ('vpc_storage_size', 'storage'),
    ('vpc_floating_ip_count', 'floating_ip_count'),
    ('vpc_instance_count', 'instances'),
)

TENANT_PATHS = {
    structure_models.Project: models.Tenant.Permissions.project_path,
    structure_models.Customer: models.Tenant.Permissions.customer_path,
}


def inject_tenant_quotas():
    for model in TENANT_PATHS:
        for (quota_name, child_quota_name) in TENANT_QUOTAS:

            def get_children(scope):
                path = TENANT_PATHS[type(scope)]
                return models.Tenant.objects.filter(**{path: scope})

            model.add_quota_field(
                name=quota_name,
                quota_field=quota_fields.UsageAggregatorQuotaField(
                    get_children=get_children,
                    child_quota_name=child_quota_name,
                )
            )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-24 07:35
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('logging', '0003_system_notification'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailhook',
            name='event_groups',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=list, verbose_name='List of event groups'),
        ),
        migrations.AlterField(
            model_name='emailhook',
            name='event_types',
            field=django.contrib.postgres.fields.jsonb.JSONField(verbose_name='List of event types'),
        ),
        migrations.AlterField(
            model_name='pushhook',
            name='event_groups',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=list, verbose_name='List of event groups'),
        ),
        migrations.AlterField(
            model_name='pushhook',
            name='event_types',
            field=django.contrib.postgres.fields.jsonb.JSONField(verbose_name='List of event types'),
        ),
        migrations.AlterField(
            model_name='systemnotification',
            name='event_groups',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=list, verbose_name='List of event groups'),
        ),
        migrations.AlterField(
            model_name='systemnotification',
            name='event_types',
            field=django.contrib.postgres.fields.jsonb.JSONField(verbose_name='List of event types'),
        ),
        migrations.AlterField(
            model_name='webhook',
            name='event_groups',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=list, verbose_name='List of event groups'),
        ),
        migrations.AlterField(
            model_name='webhook',
            name='event_types',
            field=django.contrib.postgres.fields.jsonb.JSONField(verbose_name='List of event types'),
        ),
    ]



from __future__ import unicode_literals

from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import transaction

from waldur_core.core.utils import DryRunCommand

User = get_user_model()


class Command(DryRunCommand):
    help_text = 'Append civil number with country code for OpenID users.'

    def handle(self, dry_run, *args, **options):
        conf = settings.WALDUR_AUTH_OPENID
        country_code = conf['COUNTRY_CODE']
        registration_method = conf['NAME']

        with transaction.atomic():
            users = User.objects.filter(registration_method=registration_method)\
                .exclude(civil_number__startswith=country_code)\
                .exclude(civil_number='') \
                .exclude(civil_number=None)
            count = users.count()
            for user in users:
                new_civil_number = '%s%s' % (country_code, user.civil_number)
                self.stdout.write('Username: %s, before: %s, after: %s' % (
                    user.username, user.civil_number, new_civil_number))
                if not dry_run:
                    user.civil_number = new_civil_number
                    user.save(update_fields=['civil_number'])

        self.stdout.write(self.style.SUCCESS('Civil numbers have been updated for %s users.' % count))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


def remove_all_cart_items(apps, schema_editor):
    CartItem = apps.get_model('marketplace', 'CartItem')
    CartItem.objects.all().delete()


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0009_project_is_removed'),
        ('marketplace', '0085_terms_of_service_field'),
    ]

    operations = [
        migrations.RunPython(remove_all_cart_items, elidable=True),
        migrations.AddField(
            model_name='cartitem',
            name='project',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='structure.Project'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-08-01 07:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_vmware', '0019_port_service_project_link'),
    ]

    operations = [
        migrations.AddField(
            model_name='virtualmachine',
            name='guest_power_enabled',
            field=models.BooleanField(default=False, help_text='Flag indicating if the virtual machine is ready to process soft power operations.'),
        ),
        migrations.AddField(
            model_name='virtualmachine',
            name='guest_power_state',
            field=models.CharField(blank=True, choices=[('RUNNING', 'Running'), ('SHUTTING_DOWN', 'Shutting down'), ('RESETTING', 'Resetting'), ('STANDBY', 'Standby'), ('NOT_RUNNING', 'Not running'), ('UNAVAILABLE', 'Unavailable')], max_length=150, verbose_name='The power state of the guest operating system.'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-04-26 07:49
from __future__ import unicode_literals

from django.db import migrations, models
import waldur_core.core.validators


def fill_resource_name(apps, schema_editor):
    Resource = apps.get_model('marketplace', 'Resource')

    for resource in Resource.objects.all():
        name = resource.attributes.get('name')

        if name:
            resource.name = name
            resource.save()


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0079_orderitem_old_plan'),
    ]

    operations = [
        migrations.AddField(
            model_name='resource',
            name='name',
            field=models.CharField(default='', max_length=150, validators=[waldur_core.core.validators.validate_name], verbose_name='name'),
            preserve_default=False,
        ),
        migrations.RunPython(fill_resource_name, elidable=True),
    ]



from django.apps import AppConfig


class RijkscloudConfig(AppConfig):
    name = 'waldur_rijkscloud'
    verbose_name = 'Rijkscloud'
    service_name = 'Rijkscloud'

    def ready(self):
        from waldur_core.structure import SupportedServices

        from .backend import RijkscloudBackend

        SupportedServices.register_backend(RijkscloudBackend)



from __future__ import unicode_literals

from django.core.management.base import BaseCommand, CommandError

from waldur_core.core.csv import UnicodeDictWriter

from ... import models


class Command(BaseCommand):
    help_text = "Export catalog of Amazon images."
    args = "[ami_catalog.csv]"

    def handle(self, *args, **options):
        if len(args) == 0:
            raise CommandError('AMI catalog filename is not specified.')

        writer = UnicodeDictWriter(open(args[0], 'w'), fieldnames=('name', 'region', 'backend_id'))
        writer.writeheader()
        rows = [dict(name=image.name, region=image.region.name, backend_id=image.backend_id)
                for image in models.Image.objects.all()]
        writer.writerows(rows)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django_fsm


def migrate_backups_states(apps, schema_editor):
    Backup = apps.get_model('openstack', 'Backup')
    migration_map = {
        1: 3,  # READY -> OK,
        2: 6,  # BACKING_UP -> CREATING,
        3: 3,  # RESTORING -> OK,
        4: 8,  # DELETING -> DELETING,
        5: 4,  # ERRED -> ERRED,
        6: 4,  # DELETED -> ERRED.
    }
    for backup in Backup.objects.all():
        backup.state = migration_map[backup.state]
        backup.tenant = backup.instance.tenant
        backup.save()


class Migration(migrations.Migration):

    dependencies = [
        ('openstack', '0015_instance_runtime_state'),
    ]

    operations = [
        migrations.AddField(
            model_name='backup',
            name='error_message',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='backup',
            name='tenant',
            field=models.ForeignKey(related_name='backups', to='openstack.Tenant', null=True),
        ),
        migrations.AlterField(
            model_name='backup',
            name='state',
            field=django_fsm.FSMIntegerField(default=5, choices=[(5, 'Creation Scheduled'), (6, 'Creating'), (1, 'Update Scheduled'), (2, 'Updating'), (7, 'Deletion Scheduled'), (8, 'Deleting'), (3, 'OK'), (4, 'Erred')]),
        ),
        migrations.RunPython(migrate_backups_states, elidable=True),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-06-11 11:40
from __future__ import unicode_literals

from decimal import Decimal

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('packages', '0014_unique_package_tenant'),
    ]

    operations = [
        migrations.AddField(
            model_name='packagetemplate',
            name='unit',
            field=models.CharField(choices=[('month', 'Per month'), ('half_month', 'Per half month'), ('day', 'Per day'), ('hour', 'Per hour'), ('quantity', 'Quantity')], default='day', max_length=30),
        ),
        migrations.AlterField(
            model_name='packagecomponent',
            name='price',
            field=models.DecimalField(decimal_places=10, default=0, max_digits=14, validators=[django.core.validators.MinValueValidator(Decimal('0'))], verbose_name='Price per unit'),
        ),
    ]



import importlib

from django.conf import settings


def get_active_backend():
    path = settings.WALDUR_SUPPORT['ACTIVE_BACKEND']
    module_path, class_name = path.split(':')
    module = importlib.import_module(module_path)
    klass = getattr(module, class_name)
    return klass()


class SupportBackendError(Exception):
    pass


class SupportBackend(object):
    """ Interface for support backend """

    def create_issue(self, issue):
        raise NotImplementedError

    def update_issue(self, issue):
        raise NotImplementedError

    def delete_issue(self, issue):
        raise NotImplementedError

    def create_comment(self, comment):
        raise NotImplementedError

    def update_comment(self, comment):
        raise NotImplementedError

    def delete_comment(self, comment):
        raise NotImplementedError

    def create_attachment(self, attachment):
        raise NotImplementedError

    def delete_attachment(self, attachment):
        raise NotImplementedError

    def get_users(self):
        """
        This method should return all users that are related to support project on backend.

        Each user should be represented as not saved SupportUser instance.
        """
        raise NotImplementedError

    def pull_priorities(self):
        """
        This method should pull priorities from backend and to the local database.
        """
        raise NotImplementedError



from django.db import models as django_models
from django.db.models import Q

from waldur_core.structure import models as structure_models


class SupportUserQuerySet(django_models.QuerySet):

    def get_or_create_from_user(self, user):
        """ Get or create support user based on regular user """
        return self.get_or_create(user=user, defaults={'name': user.full_name or user.username})


SupportUserManager = django_models.Manager.from_queryset(SupportUserQuerySet)


class AttachmentQuerySet(django_models.QuerySet):
    def filter_for_user(self, user):
        if not user.is_staff:
            user_customers = structure_models.Customer.objects.filter(
                permissions__role=structure_models.CustomerRole.OWNER,
                permissions__user=user,
                permissions__is_active=True)
            subquery = Q(issue__customer__in=user_customers) | Q(issue__caller=user)
            return self.filter(subquery)
        return self


AttachmentManager = django_models.Manager.from_queryset(AttachmentQuerySet)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


def pull_floating_ip_tenant(apps, schema_editor):
    FloatingIP = apps.get_model('openstack', 'FloatingIP')
    for floating_ip in FloatingIP.objects.all():
        floating_ip.tenant = floating_ip.service_project_link.tenants.first()
        floating_ip.save()


class Migration(migrations.Migration):

    dependencies = [
        ('openstack', '0010_require_security_group_tenant'),
    ]

    operations = [
        migrations.AddField(
            model_name='floatingip',
            name='tenant',
            field=models.ForeignKey(related_name='floating_ips', to='openstack.Tenant', null=True),
        ),
        migrations.RunPython(pull_floating_ip_tenant, elidable=True)
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-08 13:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_azure', '0012_network_security_group'),
    ]

    operations = [
        migrations.AddField(
            model_name='publicip',
            name='ip_address',
            field=models.GenericIPAddressField(blank=True, default=None, null=True, protocol='IPv4'),
        ),
    ]



from __future__ import unicode_literals

from django.db import IntegrityError, transaction
from django.db.models import Count, OuterRef, Subquery, F, Q, ExpressionWrapper, PositiveSmallIntegerField
from django.http import Http404, HttpResponse
from django.shortcuts import get_object_or_404
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django_filters.rest_framework import DjangoFilterBackend
from django_fsm import TransitionNotAllowed
from rest_framework import status, exceptions as rf_exceptions, viewsets as rf_viewsets
from rest_framework import views
from rest_framework.decorators import detail_route, list_route
from rest_framework.exceptions import PermissionDenied
from rest_framework.response import Response
from rest_framework.reverse import reverse

from waldur_core.core import validators as core_validators
from waldur_core.core import views as core_views
from waldur_core.core.mixins import EagerLoadMixin
from waldur_core.core.utils import order_with_nulls, month_start
from waldur_core.structure import models as structure_models
from waldur_core.structure import filters as structure_filters
from waldur_core.structure import permissions as structure_permissions
from waldur_core.structure import views as structure_views
from waldur_core.structure.permissions import _has_owner_access
from waldur_core.structure import utils as structure_utils
from waldur_core.structure.signals import resource_imported

from . import serializers, models, filters, tasks, plugins, permissions


class BaseMarketplaceView(core_views.ActionsViewSet):
    lookup_field = 'uuid'
    filter_backends = (DjangoFilterBackend,)
    update_permissions = \
        partial_update_permissions = \
        destroy_permissions = \
        [structure_permissions.is_owner]


class ServiceProviderViewSet(BaseMarketplaceView):
    queryset = models.ServiceProvider.objects.all()
    serializer_class = serializers.ServiceProviderSerializer
    filter_class = filters.ServiceProviderFilter
    api_secret_code_permissions = [structure_permissions.is_owner]

    @detail_route(methods=['GET', 'POST'])
    def api_secret_code(self, request, uuid=None):
        """ On GET request - return service provider api_secret_code.
            On POST - generate new service provider api_secret_code.
        """
        service_provider = self.get_object()
        if request.method == 'GET':
            return Response({
                'api_secret_code': service_provider.api_secret_code
            }, status=status.HTTP_200_OK)
        else:
            service_provider.generate_api_secret_code()
            service_provider.save()
            return Response({
                'detail': _('Api secret code updated.'),
                'api_secret_code': service_provider.api_secret_code
            }, status=status.HTTP_200_OK)

    def check_related_resources(request, view, obj=None):
        if obj and obj.has_active_offerings:
            raise rf_exceptions.ValidationError(_(
                'Service provider has active offerings. Please archive them first.'
            ))

    destroy_permissions = [structure_permissions.is_owner, check_related_resources]


class CategoryViewSet(EagerLoadMixin, core_views.ActionsViewSet):
    queryset = models.Category.objects.all()
    serializer_class = serializers.CategorySerializer
    lookup_field = 'uuid'
    filter_backends = (DjangoFilterBackend,)

    create_permissions = \
        update_permissions = \
        partial_update_permissions = \
        destroy_permissions = \
        [structure_permissions.is_staff]


def can_update_offering(request, view, obj=None):
    offering = obj

    if not offering:
        return

    if offering.state == models.Offering.States.DRAFT:
        structure_permissions.is_owner(request, view, offering)
    else:
        structure_permissions.is_staff(request, view)


def validate_offering_update(offering):
    if offering.state == models.Offering.States.ARCHIVED:
        raise rf_exceptions.ValidationError(_('It is not possible to update archived offering.'))


class OfferingViewSet(BaseMarketplaceView):
    queryset = models.Offering.objects.all()
    serializer_class = serializers.OfferingDetailsSerializer
    create_serializer_class = serializers.OfferingCreateSerializer
    update_serializer_class = partial_update_serializer_class = serializers.OfferingUpdateSerializer
    filter_class = filters.OfferingFilter
    filter_backends = (
        DjangoFilterBackend,
        filters.OfferingCustomersFilterBackend,
        filters.OfferingImportableFilterBackend,
        filters.ExternalOfferingFilterBackend,
    )

    @detail_route(methods=['post'])
    def activate(self, request, uuid=None):
        return self._update_state('activate')

    @detail_route(methods=['post'])
    def pause(self, request, uuid=None):
        return self._update_state('pause', request)

    pause_serializer_class = serializers.OfferingPauseSerializer

    @detail_route(methods=['post'])
    def archive(self, request, uuid=None):
        return self._update_state('archive')

    def _update_state(self, action, request=None):
        offering = self.get_object()

        try:
            getattr(offering, action)()
        except TransitionNotAllowed:
            raise rf_exceptions.ValidationError(_('Offering state is invalid.'))

        if request:
            serializer = self.get_serializer(offering, data=request.data, partial=True)
            serializer.is_valid(raise_exception=True)
            offering = serializer.save()

        offering.save(update_fields=['state'])
        return Response({
            'detail': _('Offering state updated.'),
            'state': offering.get_state_display()
        }, status=status.HTTP_200_OK)

    pause_permissions = \
        archive_permissions = \
        [structure_permissions.is_owner]

    activate_permissions = [structure_permissions.is_staff]

    activate_validators = \
        pause_validators = \
        archive_validators = \
        destroy_validators = \
        [structure_utils.check_customer_blocked]

    update_permissions = \
        partial_update_permissions = \
        [can_update_offering]

    update_validators = \
        partial_update_validators = \
        [validate_offering_update, structure_utils.check_customer_blocked]

    def perform_create(self, serializer):
        customer = serializer.validated_data['customer']
        structure_utils.check_customer_blocked(customer)

        super(OfferingViewSet, self).perform_create(serializer)

    @detail_route(methods=['get'])
    def importable_resources(self, request, uuid=None):
        offering = self.get_object()
        resources = plugins.manager.get_importable_resources(offering)

        page = self.paginate_queryset(resources)
        return self.get_paginated_response(page)

    importable_resources_permissions = [permissions.user_can_list_importable_resources]

    import_resource_permissions = [permissions.user_can_list_importable_resources]

    import_resource_serializer_class = serializers.ImportResourceSerializer

    @detail_route(methods=['post'])
    def import_resource(self, request, uuid=None):
        offering = self.get_object()

        marketplace_serializer = self.get_serializer(data=request.data)
        marketplace_serializer.is_valid(raise_exception=True)

        plan = marketplace_serializer.validated_data['plan']
        project = marketplace_serializer.validated_data['project']
        backend_id = marketplace_serializer.validated_data['backend_id']

        service_model = plugins.manager.get_service_model(offering.type)
        service = service_model.objects.get(settings=offering.scope, customer=project.customer)

        spl_model = plugins.manager.get_spl_model(offering.type)
        spl = spl_model.objects.get(project=project, service=service)
        spl_url = reverse('{}-detail'.format(spl.get_url_name()), kwargs={'pk': spl.pk})

        resource_data = {
            'backend_id': backend_id,
            'service_project_link': spl_url,
        }

        resource_viewset = plugins.manager.get_resource_viewset(offering.type)
        serializer_class = resource_viewset.import_resource_serializer_class

        serializer = serializer_class(data=resource_data, context=self.get_serializer_context())
        serializer.is_valid(raise_exception=True)

        try:
            resource = serializer.save()
        except IntegrityError:
            raise rf_exceptions.ValidationError(_('Resource is already registered.'))
        else:
            resource_imported.send(
                sender=resource.__class__,
                instance=resource,
                plan=plan,
            )

        if resource_viewset.import_resource_executor:
            transaction.on_commit(lambda: resource_viewset.import_resource_executor.execute(resource))

        marketplace_resource = models.Resource.objects.get(scope=resource)
        resource_serializer = serializers.ResourceSerializer(
            marketplace_resource, context=self.get_serializer_context())

        return Response(data=resource_serializer.data,
                        status=status.HTTP_201_CREATED)


class PlanUsageReporter(object):
    """
    This class provides aggregate counts of how many plans of a
    certain type for each offering is used.
    """
    def __init__(self, view, request):
        self.view = view
        self.request = request

    def get_report(self):
        plans = models.Plan.objects.exclude(offering__billable=False)

        query = self.parse_query()
        if query:
            plans = self.apply_filters(query, plans)

        resources = self.get_subquery()
        remaining = ExpressionWrapper(
            F('limit') - F('usage'),
            output_field=PositiveSmallIntegerField()
        )
        plans = plans.annotate(usage=Subquery(resources[:1]), limit=F('max_amount'))\
            .annotate(remaining=remaining)
        plans = self.apply_ordering(plans)

        return self.serialize(plans)

    def parse_query(self):
        if self.request.query_params:
            serializer = serializers.PlanUsageRequestSerializer(data=self.request.query_params)
            serializer.is_valid(raise_exception=True)
            return serializer.validated_data
        return None

    def get_subquery(self):
        # Aggregate
        resources = models.Resource.objects \
            .filter(plan_id=OuterRef('pk')) \
            .exclude(state=models.Resource.States.TERMINATED) \
            .annotate(count=Count('*')) \
            .values_list('count', flat=True)

        # Workaround for Django bug:
        # https://code.djangoproject.com/ticket/28296
        # It allows to remove extra GROUP BY clause from the subquery.
        resources.query.group_by = []

        return resources

    def apply_filters(self, query, plans):
        if query.get('offering_uuid'):
            plans = plans.filter(offering__uuid=query.get('offering_uuid'))

        if query.get('customer_provider_uuid'):
            plans = plans.filter(offering__customer__uuid=query.get('customer_provider_uuid'))

        return plans

    def apply_ordering(self, plans):
        param = self.request.query_params and self.request.query_params.get('o') or '-usage'
        return order_with_nulls(plans, param)

    def serialize(self, plans):
        page = self.view.paginate_queryset(plans)
        serializer = serializers.PlanUsageResponseSerializer(page, many=True)
        return self.view.get_paginated_response(serializer.data)


def validate_plan_update(plan):
    if models.Resource.objects.filter(plan=plan).exists():
        raise rf_exceptions.ValidationError(_('It is not possible to update plan because it is used by resources.'))


def validate_plan_archive(plan):
    if plan.archived:
        raise rf_exceptions.ValidationError(_('Plan is already archived.'))


class PlanViewSet(BaseMarketplaceView):
    queryset = models.Plan.objects.all()
    serializer_class = serializers.PlanDetailsSerializer
    filter_class = filters.PlanFilter

    disabled_actions = ['destroy']
    update_validators = partial_update_validators = [validate_plan_update]

    archive_permissions = [structure_permissions.is_owner]
    archive_validators = [validate_plan_archive]

    @detail_route(methods=['post'])
    def archive(self, request, uuid=None):
        plan = self.get_object()
        plan.archived = True
        plan.save(update_fields=['archived'])
        return Response({'detail': _('Plan has been archived.')}, status=status.HTTP_200_OK)

    @list_route()
    def usage_stats(self, request):
        return PlanUsageReporter(self, request).get_report()


class ScreenshotViewSet(BaseMarketplaceView):
    queryset = models.Screenshot.objects.all()
    serializer_class = serializers.ScreenshotSerializer
    filter_class = filters.ScreenshotFilter


class OrderViewSet(BaseMarketplaceView):
    queryset = models.Order.objects.all()
    serializer_class = serializers.OrderSerializer
    filter_backends = (structure_filters.GenericRoleFilter, DjangoFilterBackend)
    filter_class = filters.OrderFilter
    destroy_validators = partial_update_validators = [structure_utils.check_customer_blocked]

    @detail_route(methods=['post'])
    def approve(self, request, uuid=None):
        tasks.approve_order(self.get_object(), request.user)

        return Response({'detail': _('Order has been approved.')}, status=status.HTTP_200_OK)

    approve_validators = [core_validators.StateValidator(models.Order.States.REQUESTED_FOR_APPROVAL),
                          structure_utils.check_customer_blocked]
    approve_permissions = [permissions.user_can_approve_order_permission]

    @detail_route(methods=['post'])
    def reject(self, request, uuid=None):
        order = self.get_object()
        order.reject()
        order.save(update_fields=['state'])
        return Response({'detail': _('Order has been rejected.')}, status=status.HTTP_200_OK)

    reject_validators = [core_validators.StateValidator(models.Order.States.REQUESTED_FOR_APPROVAL),
                         structure_utils.check_customer_blocked]
    reject_permissions = [permissions.user_can_reject_order]

    @detail_route()
    def pdf(self, request, uuid=None):
        order = self.get_object()
        if not order.has_file():
            raise Http404()

        file_response = HttpResponse(order.file, content_type='application/pdf')
        filename = order.get_filename()
        file_response['Content-Disposition'] = 'attachment; filename="{filename}"'.format(filename=filename)
        return file_response

    def perform_create(self, serializer):
        project = serializer.validated_data['project']
        structure_utils.check_customer_blocked(project)

        super(OrderViewSet, self).perform_create(serializer)


class PluginViewSet(views.APIView):
    def get(self, request):
        offering_types = plugins.manager.get_offering_types()
        payload = []
        for offering_type in offering_types:
            components = [
                dict(
                    type=component.type,
                    name=component.name,
                    measured_unit=component.measured_unit,
                    billing_type=component.billing_type,
                )
                for component in plugins.manager.get_components(offering_type)
            ]
            payload.append(dict(
                offering_type=offering_type,
                components=components,
                available_limits=plugins.manager.get_available_limits(offering_type),
            ))
        return Response(payload, status=status.HTTP_200_OK)


class CustomerOfferingViewSet(views.APIView):
    serializer_class = serializers.CustomerOfferingSerializer

    def _get_customer(self, request, uuid):
        user = request.user
        if not user.is_staff:
            raise rf_exceptions.PermissionDenied()

        return get_object_or_404(structure_models.Customer, uuid=uuid)

    def get(self, request, uuid):
        customer = self._get_customer(request, uuid)
        serializer = self.serializer_class(customer, context={'request': request})
        return Response(serializer.data, status=status.HTTP_200_OK)

    def post(self, request, uuid):
        customer = self._get_customer(request, uuid)
        serializer = self.serializer_class(instance=customer, data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(status=status.HTTP_200_OK)


class OrderItemViewSet(BaseMarketplaceView):
    queryset = models.OrderItem.objects.all()
    filter_backends = (structure_filters.GenericRoleFilter, DjangoFilterBackend)
    serializer_class = serializers.OrderItemDetailsSerializer
    filter_class = filters.OrderItemFilter

    def order_items_destroy_validator(order_item):
        if not order_item:
            return
        if order_item.order.state != models.Order.States.REQUESTED_FOR_APPROVAL:
            raise rf_exceptions.PermissionDenied()

    destroy_validators = [order_items_destroy_validator]
    destroy_permissions = terminate_permissions = [structure_permissions.is_administrator]

    @detail_route(methods=['post'])
    def terminate(self, request, uuid=None):
        order_item = self.get_object()
        if not plugins.manager.can_terminate_order_item(order_item.offering.type):
            return Response({
                'details': 'Order item could not be terminated because it is not supported by plugin.'
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            # It is expected that plugin schedules Celery task to call backend
            # and then switches order item to terminated state.
            order_item.set_state_terminating()
            order_item.save(update_fields=['state'])
        except TransitionNotAllowed:
            return Response({
                'details': 'Order item could not be terminated because it has been already processed.'
            }, status=status.HTTP_400_BAD_REQUEST)
        return Response({
            'details': 'Order item termination has been scheduled.'
        }, status=status.HTTP_202_ACCEPTED)


class CartItemViewSet(core_views.ActionsViewSet):
    queryset = models.CartItem.objects.all()
    lookup_field = 'uuid'
    serializer_class = serializers.CartItemSerializer
    filter_backends = (DjangoFilterBackend,)
    filter_class = filters.CartItemFilter

    def get_queryset(self):
        return self.queryset.filter(user=self.request.user)

    @list_route(methods=['post'])
    def submit(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        order = serializer.save()
        order_serializer = serializers.OrderSerializer(instance=order, context=self.get_serializer_context())
        return Response(order_serializer.data, status=status.HTTP_201_CREATED)

    submit_serializer_class = serializers.CartSubmitSerializer


class ResourceViewSet(core_views.ReadOnlyActionsViewSet):
    queryset = models.Resource.objects.exclude(state=models.Resource.States.TERMINATED)
    filter_backends = (
        structure_filters.GenericRoleFilter,
        DjangoFilterBackend,
        filters.ResourceScopeFilterBackend
    )
    filter_class = filters.ResourceFilter
    lookup_field = 'uuid'
    serializer_class = serializers.ResourceSerializer

    @detail_route(methods=['post'])
    def terminate(self, request, uuid=None):
        resource = self.get_object()
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        attributes = serializer.validated_data.get('attributes', {})

        with transaction.atomic():
            order_item = models.OrderItem(
                resource=resource,
                offering=resource.offering,
                type=models.OrderItem.Types.TERMINATE,
                attributes=attributes,
            )
            order = serializers.create_order(
                project=resource.project,
                user=self.request.user,
                items=[order_item],
                request=request,
            )

        return Response({'order_uuid': order.uuid}, status=status.HTTP_200_OK)

    terminate_serializer_class = serializers.ResourceTerminateSerializer

    @detail_route(methods=['post'])
    def switch_plan(self, request, uuid=None):
        resource = self.get_object()
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        plan = serializer.validated_data['plan']

        with transaction.atomic():
            order_item = models.OrderItem(
                resource=resource,
                offering=resource.offering,
                old_plan=resource.plan,
                plan=plan,
                type=models.OrderItem.Types.UPDATE,
                limits=resource.limits or {},
            )
            order = serializers.create_order(
                project=resource.project,
                user=self.request.user,
                items=[order_item],
                request=request,
            )

        return Response({'order_uuid': order.uuid}, status=status.HTTP_200_OK)

    switch_plan_serializer_class = serializers.ResourceSwitchPlanSerializer

    @detail_route(methods=['post'])
    def update_limits(self, request, uuid=None):
        resource = self.get_object()
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        limits = serializer.validated_data['limits']

        with transaction.atomic():
            order_item = models.OrderItem(
                resource=resource,
                offering=resource.offering,
                plan=resource.plan,
                type=models.OrderItem.Types.UPDATE,
                limits=limits,
                attributes={'old_limits': resource.limits},
            )
            order = serializers.create_order(
                project=resource.project,
                user=self.request.user,
                items=[order_item],
                request=request,
            )

        return Response({'order_uuid': order.uuid}, status=status.HTTP_200_OK)

    update_limits_serializer_class = serializers.ResourceUpdateLimitsSerializer

    terminate_permissions = \
        switch_plan_permissions = \
        update_limits_permissions = \
        [structure_permissions.is_administrator]
    switch_plan_validators = \
        update_limits_validators = [
            core_validators.StateValidator(models.Resource.States.OK),
            structure_utils.check_customer_blocked
        ]
    terminate_validators = [
        core_validators.StateValidator(models.Resource.States.OK, models.Resource.States.ERRED),
        structure_utils.check_customer_blocked
    ]

    @detail_route(methods=['get'])
    def plan_periods(self, request, uuid=None):
        resource = self.get_object()
        qs = models.ResourcePlanPeriod.objects.filter(resource=resource)
        qs = qs.filter(Q(end=None) | Q(end__gte=month_start(timezone.now())))
        serializer = serializers.ResourcePlanPeriodSerializer(qs, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)


class CategoryComponentUsageViewSet(core_views.ReadOnlyActionsViewSet):
    queryset = models.CategoryComponentUsage.objects.all().order_by('-date', 'component__type')
    filter_backends = (DjangoFilterBackend, filters.CategoryComponentUsageScopeFilterBackend)
    filter_class = filters.CategoryComponentUsageFilter
    serializer_class = serializers.CategoryComponentUsageSerializer


class ComponentUsageViewSet(core_views.ReadOnlyActionsViewSet):
    queryset = models.ComponentUsage.objects.all().order_by('-date', 'component__type')
    filter_backends = (structure_filters.GenericRoleFilter, DjangoFilterBackend)
    filter_class = filters.ComponentUsageFilter
    serializer_class = serializers.ComponentUsageSerializer

    @list_route(methods=['post'])
    def set_usage(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        resource = serializer.validated_data['plan_period'].resource
        if not _has_owner_access(request.user, resource.offering.customer):
            raise PermissionDenied(
                _('Only staff and service provider owner is allowed '
                  'to submit usage data for marketplace resource.')
            )
        serializer.save()
        return Response(status=status.HTTP_201_CREATED)

    set_usage_serializer_class = serializers.ComponentUsageCreateSerializer


class MarketplaceAPIViewSet(rf_viewsets.ViewSet):
    """
    TODO: Move this viewset to  ComponentUsageViewSet.
    """

    permission_classes = ()
    serializer_class = serializers.ServiceProviderSignatureSerializer

    def get_validated_data(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data['data']
        dry_run = serializer.validated_data['dry_run']

        if self.action == 'set_usage':
            data_serializer = serializers.ComponentUsageCreateSerializer(data=data)
            data_serializer.is_valid(raise_exception=True)
            if not dry_run:
                data_serializer.save()

        return serializer.validated_data, dry_run

    @list_route(methods=['post'])
    @csrf_exempt
    def check_signature(self, request, *args, **kwargs):
        self.get_validated_data(request)
        return Response(status=status.HTTP_200_OK)

    @list_route(methods=['post'])
    @csrf_exempt
    def set_usage(self, request, *args, **kwargs):
        self.get_validated_data(request)
        return Response(status=status.HTTP_201_CREATED)


class OfferingFileViewSet(core_views.ActionsViewSet):
    queryset = models.OfferingFile.objects.all()
    filter_class = filters.OfferingFileFilter
    filter_backends = [DjangoFilterBackend]
    serializer_class = serializers.OfferingFileSerializer
    lookup_field = 'uuid'
    disabled_actions = ['update', 'partial_update']

    def check_create_permissions(request, view, obj=None):
        serializer = view.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        user = request.user
        offering = serializer.validated_data['offering']

        if user.is_staff or \
                (offering.customer and offering.customer.has_user(user, structure_models.CustomerRole.OWNER)):
            return

        raise rf_exceptions.PermissionDenied()

    create_permissions = [check_create_permissions]
    destroy_permissions = [structure_permissions.is_owner]


for view in (structure_views.ProjectCountersView, structure_views.CustomerCountersView):
    def inject_resources_counter(scope):
        counters = models.AggregateResourceCount.objects.filter(scope=scope).only('count', 'category')
        return {
            'marketplace_category_{}'.format(counter.category.uuid): counter.count
            for counter in counters
        }

    view.register_dynamic_counter(inject_resources_counter)



from __future__ import unicode_literals

from waldur_core.core import WaldurExtension


class AuthValimoExtension(WaldurExtension):

    class Settings:
        WALDUR_AUTH_VALIMO = {
            'URL': None,
            'AP_ID': None,
            'AP_PWD': None,
            'DNSName': '',
            'SignatureProfile': None,
            'cert_path': None,
            'key_path': None,
            'message_prefix': 'Login code:',
            'verify_ssl': False,
            'LABEL': 'Mobile ID',
            'MOBILE_PREFIX': '+372',
        }

    @staticmethod
    def get_public_settings():
        return ['LABEL', 'MOBILE_PREFIX']

    @staticmethod
    def django_app():
        return 'waldur_auth_valimo'

    @staticmethod
    def rest_urls():
        from .urls import register_in
        return register_in

    @staticmethod
    def celery_tasks():
        from datetime import timedelta
        return {
            'valimo-auth-cleanup-auth-results': {
                'task': 'waldur_auth_valimo.cleanup_auth_results',
                'schedule': timedelta(hours=1),
            },
        }



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-07-22 11:58
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0009_project_is_removed'),
        ('waldur_vmware', '0014_virtualmachine_folder'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customerfolder',
            name='customer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='structure.Customer'),
        ),
        migrations.AlterUniqueTogether(
            name='customerfolder',
            unique_together=set([('customer', 'folder')]),
        ),
    ]
